var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"./scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"./scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"./scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_creating_python_uis_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Creating Python UIs</h1></div><p>Autodesk 3ds Max ships with PySide 2.0 and includes all standard PySide modules.  PySide2 is the preferred framework for building UIs with Python in 3ds Max.  </p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> Because 3ds Max is single-threaded, the main UI is not updated while a Python script is running, even if it changes the scene, unless a UI update is requested.  For example, a new scene object will not appear in the viewport until the script exits or <code>pymxs.runtime.redrawViews()</code> is called.</p>\n\
\n\
</div></div>\n\
\n\
<h2 id=\"getting-the-max-main-window-widget\">Getting the Max Main Window Widget</h2><p>In most cases you will need a parent widget for your PySide UI, which is the 3ds Max main window.   The way to get the QWidget version of the 3ds Max main window is this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime\n\
from PySide2.QtWidgets import QWidget\n\
main_window_qwdgt = QWidget.find(runtime.windows.getMAXHWND())</code></pre>\n\
<p>Widgets parented to the main widget will behave predictably, minimizing when the main window is minimized, and deleted properly when 3ds Max exits. To display the widget as modeless, call <code>.show()</code>, and to open it as modal, call <code>.exec_()</code>.  Widgets parented to the QWidget version of the 3ds Max main window are not dockable.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> Because 3ds Max is single-threaded, the main UI is not updated while a Python script is running, even if it changes the scene, unless a UI update is requested.  For example, a new scene object will not appear in the viewport until the script exits or <code>pymxs.runtime.redrawViews()</code> is called.</p>\n\
\n\
</div></div>\n\
\n\
<h2 id=\"creating-a-dockable-widget\">Creating a Dockable Widget</h2><p>To enable docking (such as in the example below), you can use the <code>QMainWindow</code> returned by the <code>qtmax.GetMaxMainWindow()</code> convenience function.</p>\n\
<p>The following example shows how to create a dockable widget that is parented to and docks with the 3ds Max main window:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&apos;&apos;&apos;\n\
Demonstrates how to create a QDockWidget with PySide2 for use in 3ds Max\n\
&apos;&apos;&apos;\n\
\n\
from PySide2 import QtCore\n\
from PySide2 import QtGui\n\
from PySide2 import QtWidgets\n\
import qtmax\n\
\n\
from pymxs import runtime as rt\n\
\n\
def make_cylinder():\n\
    cyl = rt.Cylinder(radius=10, height=30)\n\
    rt.redrawViews()\n\
\n\
    return    \n\
\n\
class PyMaxDockWidget(QtWidgets.QDockWidget):\n\
    def __init__(self, parent=None):\n\
        super(PyMaxDockWidget, self).__init__(parent)\n\
        self.setWindowFlags(QtCore.Qt.Tool)\n\
        self.setWindowTitle(&apos;Pyside Qt  Dock Window&apos;)\n\
        self.initUI()\n\
        self.setAttribute(QtCore.Qt.WA_DeleteOnClose)\n\
\n\
    def initUI(self):\n\
        main_layout = QtWidgets.QVBoxLayout()\n\
        label = QtWidgets.QLabel(&quot;Click button to create a cylinder in the scene&quot;)\n\
        main_layout.addWidget(label)\n\
\n\
        cylinder_btn = QtWidgets.QPushButton(&quot;Cylinder&quot;)\n\
        cylinder_btn.clicked.connect(make_cylinder)\n\
        main_layout.addWidget(cylinder_btn)\n\
        widget = QtWidgets.QWidget()\n\
        widget.setLayout(main_layout)\n\
        self.setWidget(widget)\n\
        self.resize(250, 100)\n\
\n\
def main():\n\
    rt.resetMaxFile(rt.name(&apos;noPrompt&apos;))\n\
    main_window = qtmax.GetQMaxMainWindow()\n\
    w = PyMaxDockWidget(parent=main_window)\n\
    w.setFloating(True)\n\
    w.show()\n\
\n\
if __name__ == &apos;__main__&apos;:\n\
    main()</code></pre>\n\
<p>Some notes on this example: </p>\n\
<ul>\n\
<li><p>Though not strictly necessary, this example sets the window flags to <code>QtCore.Qt.Tool</code> to improve the appearance of the dialog, and sets the attribute <code>QtCore.Qt.QA_DeleteOnClose</code> to delete the dialog immediately on close.  Without this flag, the dialog is hidden, but not deleted when the close button is clicked.</p>\n\
</li>\n\
<li><p>Objects created in your script need to be protected from being garbage collected.  There are two approaches you can take.  One is to keep references to them in a static class variable.  The other is to inherit from a <code>QWidget</code> class (in this example, <code>QDockWidget</code>), which will hold the static reference, and since Qt now manages the lifecycle of the widget, it is protected from being garbage collected.</p>\n\
</li>\n\
</ul>\n\
<h2 id=\"loading-qt-ui-files\">Loading Qt UI Files</h2><p>PySide2 provides a facility for loading UI files created by Qt Designer: <code>QUiLoader</code>. This example looks for a ui file named <code>test_ui.ui</code> located in the same directory as the running script and loads it.  For more information, see <a href=\"https://doc.qt.io/qtforpython/PySide2/QtUiTools/QUiLoader.html#PySide2.QtUiTools.QUiLoader\" target=\"_blank\">QUiLoader</a>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import os\n\
from PySide2.QtWidgets import QVBoxLayout\n\
from PySide2.QtWidgets import QWidget\n\
from PySide2.QtWidgets import QDialog\n\
from PySide2.QtWidgets import QPushButton\n\
from PySide2.QtCore import QFile\n\
from PySide2.QtUiTools import QUiLoader\n\
from pymxs import runtime as rt\n\
\n\
\n\
\n\
class TestDialog(QDialog):\n\
    def __init__(self, parent=QWidget.find(rt.windows.getMAXHWND())):\n\
        QDialog.__init__(self, parent)\n\
        loader = QUiLoader()\n\
        ui_file_path = os.path.join(  os.path.dirname(os.path.realpath(__file__)), &apos;test_ui.ui&apos;)\n\
        ui_file = QFile(ui_file_path)\n\
        ui_file.open(QFile.ReadOnly)\n\
        self.ui = loader.load(ui_file, self)\n\
        ui_file.close()\n\
\n\
        layout = QVBoxLayout()\n\
        layout.addWidget(self.ui)\n\
        self.setLayout(layout)\n\
        btn = self.ui.findChild(QPushButton, &apos;pushButton&apos;)\n\
        btn.clicked.connect(self.makeTeapot)\n\
\n\
    def makeTeapot(self):\n\
        rt.teapot()\n\
        rt.redrawViews()\n\
\n\
def main():\n\
    dlg = TestDialog()\n\
    dlg.show()\n\
\n\
\n\
if __name__ == &apos;__main__&apos;:\n\
    main()</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";