var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_executing_python_executing_python_from_maxscript_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Executing Python from MAXScript</h1></div><h2 id=\"python-interface-methods\">Python interface methods</h2><p>The MAXScript Python interface contains methods for using Python in MAXScript. If you run</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">showinterface python</code></pre><p>in MAXScript, you obtain, in the MAXScript Listener, a list of available methods. The following list describes these methods and arguments:</p>\n\
<ul>\n\
<li><p><code>Init()</code>: Loads Python if it is not loaded already.</p>\n\
</li>\n\
<li><p><code>Execute()</code> and <code>ExecuteFile()</code>: Loads Python if it is not loaded already. Use <code>Init()</code> if you know that you will be using Python, and do not want the delay associated with loading it to occur the first time it is used. You would call <code>python.Init()</code> in a startup script to take the load time hit at startup, rather than while actually using 3ds Max.</p>\n\
</li>\n\
<li><p><code>Import()</code>: Imports a Python module.</p>\n\
</li>\n\
<li><p><code>Reload()</code>: Reloads a Python module, for example if it has been re-defined.</p>\n\
</li>\n\
<li><p><code>GetLastError()</code>: Returns, as a string, the error message associated with the last error encountered while running the <code>Init()</code>, <code>Execute()</code>, or <code>ExecuteFile()</code> methods.</p>\n\
</li>\n\
<li><p>Result codes (<code>enums</code>): When executing Python from MAXScript, the result codes can be the following constants:</p>\n\
</li>\n\
<li><p><code>\\#success</code></p>\n\
</li>\n\
<li><p><code>\\#pathError</code></p>\n\
</li>\n\
<li><p><code>\\#initError</code></p>\n\
</li>\n\
<li><p><code>\\#scriptFileError</code></p>\n\
</li>\n\
<li><p><code>\\#executeError</code></p>\n\
</li>\n\
<li><p><code>throwOnError</code> keyword argument: Controls whether MAXScript throws a runtime exception if an error occurs while running a method. If set to <code>true</code>, you always obtain a return result code of <code>\\#success</code> because, in all other cases, a runtime exception would be thrown. It would be up to the script writer to make the call within a try/catch in order to handle the exception.</p>\n\
</li>\n\
</ul>\n\
<h2 id=\"importing-and-using-python-libraries\">Importing and Using Python Libraries</h2><p>Python libraries can be imported and used in MAXScript, using <code>Python.Import()</code>. See the &quot;Executing Python from MAXScript&quot; topic in the <em>MAXScript Help</em> for more information.</p>\n\
<h2 id=\"limitations\">Limitations</h2><p>Because of the way Python is wrapped in MAXScript, methods and functions bound to Python execution frames will not work as expected, if at all. Some examples of calls that will not work via <code>Python.import()</code> include:</p>\n\
<ul>\n\
<li><code>sys._current_frames</code></li>\n\
<li><code>sys._get_frame</code></li>\n\
<li><code>module.globals()</code></li>\n\
<li><code>module.locals()</code></li>\n\
</ul>\n\
<h2 id=\"executing-python-from-maxscript\">Executing Python from MAXScript</h2><p>You can execute Python commands and statements (can be a single command/expression or a series of statements); or a Python script (for example, a <code>.py</code> script) from within MAXScript. Use the <code>python.Execute</code> and <code>python.ExecuteFile</code> commands respectively, as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">python.Execute &quot;print &apos;hello&apos;&quot;\n\
\n\
python.ExecuteFile &quot;demoBentCylinder.py&quot;</code></pre><p>If you provide a full path using the <code>ExecuteFile</code> command; for example, as follows; 3ds Max looks for the Python file in the path provided.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">python.ExecuteFile @&quot;C:\\Program Files\\Autodesk\\3ds Max 2015\\scripts\\Python\\demoBentCylinder.py&quot;</code></pre><p>Otherwise, 3ds Max looks for the Python script under the following directories:</p>\n\
<ul>\n\
<li>User Scripts directory</li>\n\
<li>User Startup Scripts directory</li>\n\
<li>Scripts directory</li>\n\
<li>Startup Scripts</li>\n\
<li>Windows paths specified by the <code>Path</code> environment variable. To see a list of these directories, enter <code>echo %path%</code> in your command prompt.</li>\n\
</ul>\n\
<p>For example, 3ds Max would search under the following directories: <code>userscripts\\\\python</code>, <code>userscripts\\\\startup\\\\python</code>, <code>scripts\\\\python</code>, <code>scripts\\\\startup\\\\python</code>, and subsequently the windows paths.</p>\n\
<p>The output of the results from scripts are printed to the MAXScript Listener window.</p>\n\
<p><strong>Note</strong> Python scripts run in MAXScript are not thread-safe. Python commands are always executed in the main 3ds Max thread. You should not attempt to spawn separate threads in your scripts (for example, by using the Python threading module).</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../executing_python.html\">Executing Python in 3ds Max</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";