var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"./scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"./scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"./scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_threading_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Threading in Python with 3ds Max</h1></div><p>3ds Max is a single-threaded application, which means that Python threads cannot interact with 3ds Max, either the UI or the MAXScript environment.  3ds Max is not &quot;thread safe&quot;. This means you cannot, in a thread use any pymxs calls (MAXScript runs on the main thread).</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> The <code>pymxs.mxstoken()</code> function intended to support multi-threading is deprecated, and does not actually work.</p>\n\
\n\
</div></div>\n\
\n\
<p>However, threading can still be used in your scripts for accomplishing tasks that would benefit from concurrency and/or parallelism, such as file IO.  Both the standard <code>threading</code> module and <code>async</code> (in Python 3) can be used.  </p>\n\
<h2 id=\"usefulness-of-multi-threading\">Usefulness of multi threading</h2><p>There are two main reasons to use multi threading in a program: </p>\n\
<ul>\n\
<li>to make good use of multiple CPUs (cores), </li>\n\
<li>and to do concurrent IO: multiple independent IO operations at the same time instead of sequentially.</li>\n\
</ul>\n\
<h3 id=\"making-use-of-multiple-cores\">Making Use of Multiple Cores</h3><p>The way the Python Global Interpreter Lock (GIL) works makes it impossible for two python instructions to run at\n\
the same time on the same interpreter, no matter the number of cores available.\n\
The consequence of this is that python code executing tight computation loops,\n\
no matter the number of threads at work, will only keep the equivalent of one\n\
core busy. </p>\n\
<p>Heavy lifting code written in python does not benefit from multi threading, so\n\
what does?  Sometimes, python programs call functions that were not implemented\n\
in python but in a different language (ex: C). The non-Python implementation of\n\
these library functions will typically release the GIL during their execution\n\
allowing threads to run normally, so that these native parts can be executed in\n\
parallel on multiple cores. So in essence, if the heavy lifting is initiated\n\
from Python but performed in another language (after the GIL has been released)\n\
the code may benefit from multiple cores. A good example of this scenario is <code>SciPy</code> (<a href=\"https://scipy-cookbook.readthedocs.io/items/Multithreading.html\" target=\"_blank\">see details\n\
here</a>).</p>\n\
<p>It is possible for native\n\
library functions to make good use of multiple cores (and GPUS, multiple\n\
machines, or any hardware available for that matter) even if the python code\n\
that makes the calls is single threaded. If for example a python program uses\n\
TensorFlow, this library will make good use of all the available hardware to\n\
operate in an efficient way ((<a href=\"https://www.tensorflow.org/guide/distributed_training\" target=\"_blank\">explanations\n\
here</a>).  It&#39;s the\n\
implementation of a given lengthy function that will distribute the work to the\n\
available hardware so that maximum parallelism can be achieved during the\n\
execution of this call.</p>\n\
<p>In short, what&#39;s important to understand is that the multi-threading in python\n\
may help to make better use of cores but only under very particular\n\
circumstances, and that some library functions are able to do that anyway\n\
without any help from python threads. In all situations, it depends on the\n\
internal implementation of external libraries.</p>\n\
<h2 id=\"using-threads-for-concurrent-io\">Using Threads for Concurrent IO</h2><p>In most situations, performing independent IO operations concurrently takes less time\n\
than performing them sequentially, and this is the rationale for concurrent\n\
IO operations.</p>\n\
<p>Reading data from devices (such as a network or hard drive) or writing data to external\n\
devices takes time but does not require a lot of work from the CPU. When\n\
possible, it is beneficial to initiate multiple IO operations at the same time\n\
and process the data as it arrives. Normal (synchronous) IO functions block the\n\
calling thread until the IO operation completes. For that reason, initiating\n\
concurrent IO operations with blocking functions requires the use of multiple\n\
threads (the main thread launches multiple threads that each perform one\n\
blocking io operation).</p>\n\
<p>In 3ds Max, Python threads are useful in this scenario. As long as the worker\n\
threads don&#39;t use <code>pymxs</code> (or objects created with <code>pymxs</code>) this method can be used in 3ds Max to improve file IO performance.</p>\n\
<h2 id=\"using-code-async-code-for-asynchronous-io\">Using <code>async</code> for Asynchronous IO</h2><p>Threads can be used for achieving concurrent IO with blocking functions.\n\
Python 3 also includes the <code>asyncio</code> library, which allows you to do the same thing without having\n\
to pay for the memory overhead and context switching of threads. Everything is\n\
executed on the same thread through the use of an event loop (using cooperative\n\
scheduling).</p>\n\
<p>The <code>asyncio</code> library does not require threads, but it is possible to use threads with <code>asyncio</code>.\n\
For example, it is possible to expose an operation running on a thread or a\n\
process as a future (see details in <code>concurrent.futures</code>).</p>\n\
<p>In 3ds Max, it is possible to use <code>asyncio</code> to perform concurrent IO without creating \n\
threads or processes. In this scenario only one thread is used so the Python \n\
code <em>can</em> (but shouldn&#39;t) call into <code>pymxs</code>, because pymxs\n\
operations can themselves use blocking IO, and since these functions are not clearly\n\
identified and not awaitable, they will introduce synchronous IO to the mix.<br>This reduces the usefulness of asyncio, where every lengthy operation should be non-blocking. </p>\n\
<p>To play well with asyncio, pymxs would need to return its blocking\n\
functions as futures, but this is not currently the case.</p>\n\
<p>In python there are mechanisms that allow you to use thread-based operations or\n\
process-based operations as async operations. Calling from python threads\n\
into pymxs functions is not supported, no matter through which path this happens.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";