var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"./scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"./scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"./scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_troubleshooting_3ds_max_python_a_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Troubleshooting 3ds Max Python API</h1></div><h2 id=\"why-doesn-39-t-my-cpython-extension-module-load-\">Why doesn&apos;t my CPython extension module load?</h2><p>Check if your extension module was built and linked against a different version of Python. In order to use a CPython module with Python in 3ds Max, you must compile the CPython binaries against Python version 3.7. See <a href=\"python_extension_libraries.html\">Python extension libraries</a> for more information.</p>\n\
<h2 id=\"i-am-trying-to-install-an-extension-module-but-3ds-max-python-is-not-detected-by-the-installer\">I am trying to install an extension module but 3ds Max Python is not detected by the installer</h2><p>Update the registry or manually install the module. Often modules support installation via manual steps or a Python module (for example, the Python &quot;easy_install&quot; module (<a href=\"http://pythonhosted.org/distribute/easy_install.html\" target=\"_blank\">http://pythonhosted.org/distribute/easy_install.html</a>). See also <a href=\"python_extension_libraries.html\">Python extension libraries</a> for more information.</p>\n\
<h2 id=\"i-set-a-property-but-do-not-see-the-expected-result\">I set a property but do not see the expected result</h2><p>In Python, if you misspell a property when assigning it, a new field is added to the object as a result - for example: MyClass.fOotball = x, or worse yet: MyClass.GetValue = 42. It is legal to add or replace fields on an object at run-time; therefore no error occurs. The safest way to set a property is to use a <code>Set</code> function, for example: render.SetWidth(640).</p>\n\
<h2 id=\"i-run-my-script-and-it-works-however-when-i-restart-3ds-max-and-run-the-script-again-i-get-an-error-\">I run my script and it works. However, when I restart 3ds Max and run the script again, I get an error.</h2><p>When you run any Python code that affects the global environment (for example, importing a module), those settings persist during your session of 3ds Max. A common mistake is to rely on a module that was imported by another script, or to call a function that was created by another script.</p>\n\
<h2 id=\"i-have-a-script-for-example-quot-mylibrarymodule-quot-that-is-imported-by-another-script-for-example-quot-myprogram-but-after-i-make-changes-to-mylibrarymodule-the-changes-do-not-seem-to-take-effect-the-next-time-i-run-quot-myprogram-quot-\">I have a script (for example, &quot;MyLibraryModule&quot;) that is imported by another script (for example, &quot;MyProgram), but after I make changes to MyLibraryModule, the changes do not seem to take effect the next time I run &quot;MyProgram&quot;.</h2><p>Use the <code>reload(MyLibraryModule)</code> command to force Python to reload that module.</p>\n\
<h2 id=\"common-issues-and-their-workarounds\">Common issues and their workarounds</h2><ul>\n\
<li><p>When <code>sys.exit()</code> is used to exit Python, it raises a <code>SystemExit</code> exception. If an uncaught exception is raised in Python, it is reported to the MAXScript Listener window as an error, even though no error actually occurred. To avoid having an error being reported in the MAXScript Listener window by <code>sys.exit()</code>, you can use the <code>try</code> and <code>except</code> statements as follows:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">try:\n\
    import sys\n\
    print \\&apos;goodbye world\\&apos;\n\
    sys.exit() # Safe way to leave program at any point and allow objects and resources to be cleaned up. \n\
# Prevent 3ds Max from reporting a system exit as an error in the listener. \n\
except SystemExit:\n\
    pass </code></pre></li>\n\
<li><p>If you import a module in a script, the module remains &quot;imported&quot; even after you finish executing your script and start executing a different script. Any names added to the global namespace will also remain from script execution to script execution. These only persist per session of 3ds Max, however, and you can workaround this by exiting and reloading 3ds Max.</p>\n\
</li>\n\
<li><p>When creating an instance of <code>QApplication()</code>, do not use <code>app = QApplication([])</code>, use instead:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">app = PySide2.QtCore.QCoreApplication.instance()\n\
if not app:\n\
    ...</code></pre><p>to avoid a potential crash.</p>\n\
<p><strong>Note</strong> Troubleshooting tips for PySide2 also apply to PyQt.</p>\n\
</li>\n\
<li><p>Some modules require that you set <code>sys.argv = [&apos;&apos;]</code> before you use them; for example, <code>app = QtGui.QApplication(sys.argv)</code>.</p>\n\
</li>\n\
<li><p>Module names (for example, <code>pymxs</code>) and built-in functions (for example, <code>dir</code>) can be re-assigned to anything. This is especially a problem with global variables since it can break other scripts, but is acceptable within the scope of a function.</p>\n\
</li>\n\
<li><p>The <code>sys.path</code> does not contain the executing script path.  This is by design.  To get the path of the currently executing script, use the <code>__file__</code> constant.</p>\n\
</li>\n\
</ul>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";