var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_tutorials_adding_numpy_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Tutorial 2 - Adding NumPy, an external component</h1></div><p>In this tutorial we add the <a href=\"https://numpy.org/\" target=\"_blank\">NumPy</a> module to the pyramid project created in\n\
<a href=\"./creating_the_dialog.html\">Tutorial 1</a>.  This tutorial requires that the previous one is completed.</p>\n\
<p>The code of the completed tutorial can be found <a href=\"#full-code\">at the end of this topic</a>\n\
for reference.</p>\n\
<h2 id=\"installing-numpy\">Installing NumPy</h2><p>We will, in another tutorial, package our pyramid component as a pip package\n\
and this will allow us to declare its dependencies and automatically install\n\
them.</p>\n\
<p>But at this point the pyramid project is just a bunch of files in a directory and\n\
we need to install its dependencies manually.</p>\n\
<p>First, install numpy by running:\n\
<code>.\\python.exe -m pip install numpy</code> in your max Python37 installation </p>\n\
<p><em>(full details <a href=\"./../python_extension_libraries.html\">here</a>.)</em></p>\n\
<h2 id=\"a-note-on-pip\">A Note on Pip</h2><p>The installation of pip for 3ds Max is explained <a href=\"./../python_extension_libraries.html\">here</a>.</p>\n\
<p>When pip is installed with --user, it ends\n\
up in <code>/Users/myusername/AppData/Roaming/Python/Python37/Scripts/pip</code> so\n\
when we say that after installing pip we want to make sure we use this version by using \n\
pip in a path that looks like this:</p>\n\
<p><code>/Users/myusername/AppData/Roaming/Python/Python37/Scripts/pip install --user something</code> </p>\n\
<p>If we don&apos;t use the complete path, there can be other pip installations on the system and \n\
<code>pip install</code> could invoke one of these and either fail or end up at a different location.</p>\n\
<p>When packages are installed with --user for the Python37 3ds Max interpreter, they\n\
should end up in <code>/Users/myusername/AppData/Roaming/Python/Python37/site-packages</code>\n\
otherwise it will be impossible to import them.</p>\n\
<p>Ultimately what&apos;s important is to use the right pip, the one installed with\n\
3ds Max.</p>\n\
<h2 id=\"using-numpy-in-our-project\">Using NumPy in our Project</h2><p>We will add this to <em>graphics.py</em>:</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> what we do here with numpy is not very useful, the goal is to keep\n\
the sample very small.</div>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&quot;&quot;&quot;\n\
    Provide the graphic functionality for the pryamid tool.\n\
&quot;&quot;&quot;\n\
from pymxs import runtime as rt\n\
import numpy as np\n\
\n\
def make_pyramid_mesh(side=20.0):\n\
    &apos;&apos;&apos;Construct a pyramid from vertices and faces.&apos;&apos;&apos;\n\
    halfside = side / 2.0\n\
\n\
    npa = np.array([\n\
        [0.0, 0.0, side],\n\
        [-halfside, -halfside, 0.0],\n\
        [-halfside, halfside, 0.0],\n\
        [halfside, 0.0, 0.0]])\n\
\n\
    pyr = rt.mesh(\n\
        vertices=list(map(lambda l: rt.point3(*l.tolist()), npa)),\n\
        faces=[\n\
            rt.point3(1, 2, 3),\n\
            rt.point3(1, 3, 4),\n\
            rt.point3(1, 4, 2),\n\
            rt.point3(2, 3, 4),\n\
        ])\n\
    rt.redrawViews()\n\
    return pyr</code></pre>\n\
<h2 id=\"load-the-tutorial-if-we-closed-3ds-max-between-the-two-tutorials-\">Load the tutorial (If we closed 3ds Max between the two tutorials)</h2><p>Enter these lines on the listener - all lines need to be entered as separate lines followed by <strong>Ctrl+Enter</strong>.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import sys\n\
sys.path += [ &quot;d:\\\\sources\\\\hacking\\\\python\\\\pyramid&quot; ]\n\
import pyramid.ui</code></pre>\n\
<p>Or if the code is already imported from the previous tutorial:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">importlib.reload(pyramid.graphics)\n\
importlib.reload(pyramid.ui)</code></pre>\n\
<h2 id=\"run-the-tutorial\">Run the tutorial</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">dialog = pyramid.ui.PyMaxDialog()\n\
dialog.show()</code></pre>\n\
<p>We have now modified our pyramid sample to use NumPy. In the <a href=\"./packaging_pyramid.html\">next tutorial</a>\n\
we will show how to package it with pip.</p>\n\
<h2 id=\"full-code\">Full Code</h2><p><strong>graphics.py</strong>:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">\n\
&quot;&quot;&quot;\n\
    Provide the graphic functionality for the pryamid tool.\n\
&quot;&quot;&quot;\n\
from pymxs import runtime as rt\n\
import numpy as np\n\
\n\
def make_pyramid_mesh(side=20.0):\n\
    &apos;&apos;&apos;Construct a pyramid from vertices and faces.&apos;&apos;&apos;\n\
    halfside = side / 2.0\n\
\n\
    npa = np.array([\n\
        [0.0, 0.0, side],\n\
        [-halfside, -halfside, 0.0],\n\
        [-halfside, halfside, 0.0],\n\
        [halfside, 0.0, 0.0]])\n\
\n\
    pyr = rt.mesh(\n\
        vertices=list(map(lambda l: rt.point3(*l.tolist()), npa)),\n\
        faces=[\n\
            rt.point3(1, 2, 3),\n\
            rt.point3(1, 3, 4),\n\
            rt.point3(1, 4, 2),\n\
            rt.point3(2, 3, 4),\n\
        ])\n\
    rt.redrawViews()\n\
    return pyr</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../tutorials.html\">Tutorials</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";