var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_tutorials_creating_the_dialog_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/Max-Python-API/scripts/prettify/lang-txt.js\"></script>\n\
      <script src=\"/cloudhelp/2022/ENU/Max-Python-API/scripts/prettify/lang-json.js\"></script>\n\
      <script src=\"/cloudhelp/2022/ENU/Max-Python-API/scripts/prettify/lang-python.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Tutorial 1: Creating and Debugging a UI</h1></div><p>In this tutorial we will create and debug a small PySide2 tool for 3ds Max\n\
called &quot;pyramid&quot;, using Visual Studio Code and the Python module <code>ptvsd</code> (Python Tools for Visual Studio Debugger). This tool shows a Qt dialog with a button. Pressing the\n\
button adds a black pyramid to the current scene.  </p>\n\
<p>The code of the completed tutorial can be found <a href=\"#full-code\">at the end of this topic</a>\n\
for reference.</p>\n\
<h2 id=\"prerequisites\">Prerequisites</h2><p>This tutorial series requires that:</p>\n\
<ul>\n\
<li>3ds Max 2021 or later is installed</li>\n\
<li>3ds Max 2021 or later is started and using the Python 3 interpreter (the default setting)</li>\n\
<li><a href=\"https://code.visualstudio.com\" target=\"_blank\">VS Code</a> is installed, and the <a href=\"https://code.visualstudio.com/docs/languages/python\" target=\"_blank\">Python extension</a> is installed</li>\n\
<li><code>pip</code> is installed for 3ds Max Python (see <a href=\"./../python_extension_libraries.html\">Using pip with Python 3</a>)</li>\n\
</ul>\n\
<h2 id=\"creating-the-project\">Creating the project</h2><p>This project will be called pyramid. <a href=\"https://www.python.org/dev/peps/pep-0008/\" target=\"_blank\">PEP-8</a> (the Style Guide for Python) suggests that\n\
modules have small lowercase names.</p>\n\
<p>So somewhere in our file system, create a nested set of directories, both with the name &apos;pyramid&apos;:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">mkdir -p pyramid/pyramid</code></pre><p>We will eventually use this directory structure to add proper pip packaging\n\
information to our project. For this tutorial we will just create our program\n\
in it.</p>\n\
<h2 id=\"starting-vs-code\">Starting VS Code</h2><ul>\n\
<li>Start vs code</li>\n\
<li>Select our top-level <code>pyramid</code> folder with <strong>File &gt; Open Folder</strong></li>\n\
</ul>\n\
<h2 id=\"creating-the-source-files\">Creating the source files</h2><p>Our project is small, but we will separate it in two files to simulate a larger project.</p>\n\
<p>Create a new file named <em>ui.py</em> in the <strong>pyramid</strong> subdirectory, and then add the following code:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&quot;&quot;&quot;\n\
    Provide a PySide2 dialog for the pyramid tool.\n\
&quot;&quot;&quot;\n\
from PySide2.QtWidgets import QWidget, QDialog, QLabel, QVBoxLayout, QPushButton\n\
from pymxs import runtime as rt\n\
from .graphics import make_pyramid_mesh\n\
\n\
class PyMaxDialog(QDialog):\n\
    &quot;&quot;&quot;\n\
    Custom dialog attached to the 3ds Max main window\n\
    Message label and action push button to create a pyramid in the 3ds \n\
    Max scene graph\n\
    &quot;&quot;&quot;\n\
    def __init__(self, parent=QWidget.find(rt.windows.getMAXHWND())):\n\
        super(PyMaxDialog, self).__init__(parent)\n\
        self.setWindowTitle(&apos;Pyside2 Qt Window&apos;)\n\
        self.init_ui()\n\
\n\
    def init_ui(self):\n\
        &quot;&quot;&quot; Prepare Qt UI layout for custom dialog &quot;&quot;&quot;\n\
        main_layout = QVBoxLayout()\n\
        label = QLabel(&quot;Click button to create a pyramid in the scene&quot;)\n\
        main_layout.addWidget(label)\n\
\n\
        btn = QPushButton(&quot;Pyramid&quot;)\n\
        btn.clicked.connect(make_pyramid_mesh)\n\
        main_layout.addWidget(btn)\n\
\n\
        self.setLayout(main_layout)\n\
        self.resize(250, 100)\n\
</code></pre>\n\
<ul>\n\
<li>Create another file named <em>graphics.py</em> in the same directory, and add this code:</li>\n\
</ul>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&quot;&quot;&quot;\n\
    Provide the graphic functionality for the pryamid tool.\n\
&quot;&quot;&quot;\n\
from pymxs import runtime as rt\n\
\n\
def make_pyramid_mesh(side=20.0):\n\
    &apos;&apos;&apos;Construct a pyramid from vertices and faces.&apos;&apos;&apos;\n\
    halfside = side / 2.0\n\
    pyr = rt.mesh(\n\
        vertices=[\n\
            rt.point3(0.0, 0.0, side),\n\
            rt.point3(-halfside, -halfside, 0.0),\n\
            rt.point3(-halfside, halfside, 0.0),\n\
            rt.point3(halfside, 0.0, 0.0)\n\
            ],\n\
        faces=[\n\
            rt.point3(1, 2, 3),\n\
            rt.point3(1, 3, 4),\n\
            rt.point3(1, 4, 2),\n\
            rt.point3(2, 3, 4),\n\
        ])\n\
    rt.redrawViews()\n\
    return pyr</code></pre>\n\
<p>At this point you should have this following directory structure:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-txt prettyprint\">pyramid/\n\
    pyramid/\n\
        ui.py\n\
        graphics.py</code></pre>\n\
<h2 id=\"wait-i-see-problems-with-my-import-statements\">Wait, I see problems with my import statements</h2><p>Visual Studio Code uses a linter to verify imports, and by default it does not know where to find the 3ds Max Python modules. The autocomplete system can also have problems understanding our\n\
sources.</p>\n\
<p>To solve this issue, point Visual Studio Code to the 3ds Max Python interpreter:</p>\n\
<ul>\n\
<li>Open settings with <strong>File &gt; Preferences &gt; Settings</strong></li>\n\
<li>In the settings page, open <strong>Extensions &gt; Python &gt; Auto Complete: Extra Paths &gt; Edit in settings.json</strong>.  This opens the <em>settings.json</em> file.</li>\n\
<li>Add:</li>\n\
</ul>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-json prettyprint\">    ,\n\
    &quot;python.pythonPath&quot;: &quot;C:\\\\Program Files\\\\Autodesk\\\\3ds Max 2021\\\\Python37\\\\python.exe&quot;,\n\
    &quot;python.autoComplete.extraPaths&quot;: [         \n\
        &quot;C:\\\\Program Files\\\\Autodesk\\\\3ds Max 2021\\\\Python37&quot;     \n\
    ] \n\
</code></pre>\n\
<p>More information on this can be found <a href=\"https://code.visualstudio.com/docs/python/environments\" target=\"_blank\">here</a>\n\
and <a href=\"https://code.visualstudio.com/docs/python/editing\" target=\"_blank\">here</a>.</p>\n\
<h2 id=\"running-this-first-version-of-the-project-in-3ds-max\">Running this first version of the project in 3ds Max</h2><p>Switch to 3ds Max, open the listener window and select the Python tab.\n\
At that point we should see that Python 3 is used, from the version string displayed in the listener.</p>\n\
<p>By default, the 3ds Max Python interpreter does not know where our sample is, but we want to be able\n\
to make it available through <code>import</code>. So we can type this in the python\n\
listener (substituting your own path for the append function):</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import sys\n\
sys.path.append(r&apos;d:\\sources\\hacking\\python\\pyramid&apos;)</code></pre>\n\
<div class=\"note note:\"><strong><span class=\"notetitle\">Note:</span></strong> Enter each line individually, and use <strong>Ctrl+Enter</strong> to execute them.</div>\n\
<p>Then type:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import pyramid.ui\n\
dialog = pyramid.ui.PyMaxDialog()\n\
dialog.show()</code></pre>\n\
<p>this should display our dialog:\n\
<img src=\"./images/tutorials/Dialog.png\" alt=\"Dialog\"></p>\n\
<p>If you press the <strong>Pyramid</strong> button, you get an execption:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">&gt;&gt;&gt; Traceback (most recent call last):\n\
  File &quot;d:\\sources\\hacking\\python\\pyramid\\pyramid\\graphics.py&quot;, line 11, in make_pyramid_mesh\n\
    rt.point3(0.0, 0.0, side),\n\
RuntimeError: MAXScript exception raised.\n\
-- Unable to convert: false to type: Float</code></pre><p>Now we need to debug this problem.</p>\n\
<h2 id=\"debugging-3ds-max-python-code-from-vs-code\">Debugging 3ds Max Python code from VS Code</h2><p>Before we can use the debugger to debug python code in 3ds Max, we need to install the \n\
<a href=\"https://github.com/microsoft/ptvsd\" target=\"_blank\"><code>ptvsd</code></a> (Python Tools for Visual Studio Debug Server):</p>\n\
<ul>\n\
<li>In your 3ds Max Python 3 directory, in a command line, type: <code>.python.exe -m pip install --user ptvsd</code>\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> <code>pip</code> should already be installed, see the prerequisites above.</p>\n\
\n\
</div></div>\n\
\n\
</li>\n\
</ul>\n\
<h3 id=\"enabling-the-debugger-on-the-3ds-max-side\">Enabling the debugger on the 3ds Max side</h3><p>The ptvsd debug server needs to be running in the 3ds Max Python interpreter. In the 3ds Max Listener window, type:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import ptvsd\n\
ptvsd.enable_attach()</code></pre>\n\
<p>this should display:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-txt prettyprint\">(&apos;0.0.0.0&apos;, 5678)</code></pre>\n\
<p>(The previous step will not have to be repeated in our 3ds Max session).</p>\n\
<h3 id=\"connecting-with-vs-code-the-first-time\">Connecting with VS Code: the first time</h3><p>3ds Max is now ready to accept a connection from VS Code, but we must still\n\
tell VS Code how to connect to 3ds Max. The first time we want to connect to 3ds Max we need\n\
to add a configuration for remote debugging in VS Code.</p>\n\
<ul>\n\
<li>In VS Code, select <strong>Run &gt; Add Configuration</strong>.</li>\n\
<li>From the drop down list, select  <strong>Remote Attach</strong></li>\n\
<li>Then hit enter to choose  <strong>localhost</strong> (the default)</li>\n\
<li>And then hit enter to choose <strong>5678</strong> (the default)</li>\n\
</ul>\n\
<p>This will add a configuration code that looks like this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-json prettyprint\">        {\n\
            &quot;name&quot;: &quot;Python: Remote Attach&quot;,\n\
            &quot;type&quot;: &quot;python&quot;,\n\
            &quot;request&quot;: &quot;attach&quot;,\n\
            &quot;connect&quot;: {\n\
                &quot;host&quot;: &quot;localhost&quot;,\n\
                &quot;port&quot;: 5678\n\
            },\n\
            &quot;pathMappings&quot;: [\n\
                {\n\
                    &quot;localRoot&quot;: &quot;${workspaceFolder}&quot;,\n\
                    &quot;remoteRoot&quot;: &quot;.&quot;\n\
                }\n\
            ]\n\
        }</code></pre>\n\
<p>We don&apos;t need to modify this other than we will want to point the <code>remoteRoot</code> setting\n\
directly at our sources</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-json prettyprint\">                    &quot;remoteRoot&quot;: &quot;${workspaceFolder}&quot;,</code></pre>\n\
<p>This configuration only needs to be set up once.</p>\n\
<p>If we want to debug third party modules as well (for example, put breakpoints in\n\
pyside2 code), this is not something we will do in this sample but still a very\n\
common debugging task, we will also have to enable it in this json file\n\
by adding:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-json prettyprint\">&quot;justMyCode&quot;: false</code></pre>\n\
<p>In the end your configuration should look like this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-json prettyprint\">        {\n\
            &quot;name&quot;: &quot;Python: Remote Attach&quot;,\n\
            &quot;type&quot;: &quot;python&quot;,\n\
            &quot;request&quot;: &quot;attach&quot;,\n\
            &quot;justMyCode&quot;: false,\n\
            &quot;connect&quot;: {\n\
                &quot;host&quot;: &quot;localhost&quot;,\n\
                &quot;port&quot;: 5678\n\
            },\n\
            &quot;pathMappings&quot;: [\n\
                {\n\
                    &quot;localRoot&quot;: &quot;${workspaceFolder}&quot;,\n\
                    &quot;remoteRoot&quot;: &quot;${workspaceFolder}&quot;\n\
                }\n\
            ]\n\
        }</code></pre>\n\
<h3 id=\"connecting-with-vs-code\">Connecting with VS Code</h3><p>First open the Run and Debug view by selecting <strong>View &gt; Open View...</strong> and selecting &quot;Run and Debug&quot;.</p>\n\
<p>We need to select our Debug Config in VS Code:</p>\n\
<p><img src=\"./images/tutorials/DebugConfig.png\" alt=\"Debug Config\"></p>\n\
<p>Ensure that our <code>Python: Remote Attach</code> configuration is selected.  When this is the case we can connect the debugger to 3ds Max Python  by selecting \n\
<strong>Debug &gt; Start Debugging</strong> from the VS Code menu.</p>\n\
<p>If we go back to 3ds Max, we see that the UI is no longer frozen. However, our\n\
program is still not running.</p>\n\
<h3 id=\"running-the-program\">Running the program</h3><p>To run our script we need to type, in the 3ds Max listener window:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-python prettyprint\">dialog.show()</code></pre>\n\
<p>This will run our dialog that does not work the way we want.</p>\n\
<p><img src=\"./images/tutorials/Dialog.png\" alt=\"Dialog\"></p>\n\
<h3 id=\"adding-a-breakpoint\">Adding a breakpoint</h3><p>We know, from a previous run of our program that there is a problem on\n\
line 11 of <em>graphics.py</em>. So in VS Code we can add a breakpoint on this line by clicking the margin at the left of the line:</p>\n\
<p><img src=\"./images/tutorials/Breakpoint.png\" alt=\"Breakpoint\"></p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<div class=\"note note:\"><strong>Note</strong>: if this does not work, it could be that your <code>remoteRoot</code> path mapping\n\
is wrong, or that something else in the previous steps did not go well\n\
(for example, your debugger is not running).</div>\n\
\n\
</div></div>\n\
\n\
<h3 id=\"triggering-the-breakpoint\">Triggering the breakpoint</h3><p>At this point when you click on the <code>Pyramid</code> button of your dialog, your\n\
breakpoint will be reached and you will see it highlighted in VS Code.</p>\n\
<p><img src=\"./images/tutorials/BreakpointReached.png\" alt=\"BreakpointReached\"></p>\n\
<p>You will be able to inspect the locals and you will see that the <code>side</code> variable is <code>False</code>,\n\
which is not what we want.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<div class=\"note note:\"><strong>Note</strong>: It is important that you use a dialog that was started during the\n\
debugging session. If you try to click the <code>Pyramid</code> button on a dialog that\n\
was started before that, it may not work as expected.</div>\n\
\n\
</div></div>\n\
\n\
<h3 id=\"fixing-the-code\">Fixing the code</h3><p>What has happened is that <code>side</code> is <code>False</code>. Our code that connects <code>make_pyramid_mesh()</code>\n\
to the dialog button looks like this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-python prettyprint\">        btn.clicked.connect(make_pyramid_mesh)</code></pre>\n\
<p>This is wrong because <code>btn.clicked</code> passes us a Bool that tells us whether the button\n\
is checked. Our code receives this bool instead of the default <code>20.0</code> value\n\
that we expected.</p>\n\
<p>So we can edit this code to pass a lambda function that sets <code>side</code> to 20.0 instead:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-python prettyprint\">        btn.clicked.connect(lambda: make_pyramid_mesh(20.0))</code></pre>\n\
<p>And then we save</p>\n\
<h3 id=\"making-the-new-code-run\">Making the new code run</h3><p>To give the control back to max we press <code>F5</code> (Run -&gt; Continue). We see\n\
that 3ds Max becomes responsive again.</p>\n\
<ul>\n\
<li>Close the dialog</li>\n\
<li>At the listener window&apos;s python command prompt  type</li>\n\
</ul>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-python prettyprint\">import importlib</code></pre>\n\
<p>and then</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-python prettyprint\">importlib.reload(pyramid.ui)</code></pre>\n\
<p>This displays something like:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">&lt;module &apos;pyramid.ui&apos; from &apos;d:\\\\sources\\\\hacking\\\\python\\\\pyramid\\\\pyramid\\\\ui.py&apos;&gt;</code></pre><ul>\n\
<li>we must now recreate a brand new version of the dialog by doing:</li>\n\
</ul>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-python prettyprint\">dialog = pyramid.ui.PyMaxDialog()</code></pre>\n\
<ul>\n\
<li>and then:</li>\n\
<li><pre class=\"pre codeblock prettyprint\"><code class=\"language-python prettyprint\">dialog.show()</code></pre>\n\
</li>\n\
</ul>\n\
<p>Our breakpoint is reached, but this time <code>side = 20</code> and <code>halfside = 10</code> and\n\
if we hit <code>F5</code> the code will now successfully complete and show our \n\
black pyramid in the viewport.</p>\n\
<p><img src=\"./images/tutorials/Result.png\" alt=\"Result\"></p>\n\
<hr>\n\
<h2 id=\"full-code\">Full Code</h2><p><strong>ui.py</strong>:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&quot;&quot;&quot;\n\
    Provide a PySide2 dialog for the pyramid tool.\n\
&quot;&quot;&quot;\n\
from PySide2.QtWidgets import QWidget, QDialog, QLabel, QVBoxLayout, QPushButton\n\
from pymxs import runtime as rt\n\
from .graphics import make_pyramid_mesh\n\
\n\
class PyMaxDialog(QDialog):\n\
    &quot;&quot;&quot;\n\
    Custom dialog attached to the 3ds Max main window\n\
    Message label and action push button to create a pyramid in the 3ds \n\
    Max scene graph\n\
    &quot;&quot;&quot;\n\
    def __init__(self, parent=QWidget.find(rt.windows.getMAXHWND())):\n\
        super(PyMaxDialog, self).__init__(parent)\n\
        self.setWindowTitle(&apos;Pyside2 Qt Window&apos;)\n\
        self.init_ui()\n\
\n\
    def init_ui(self):\n\
        &quot;&quot;&quot; Prepare Qt UI layout for custom dialog &quot;&quot;&quot;\n\
        main_layout = QVBoxLayout()\n\
        label = QLabel(&quot;Click button to create a pyramid in the scene&quot;)\n\
        main_layout.addWidget(label)\n\
\n\
        btn = QPushButton(&quot;Pyramid&quot;)\n\
        btn.clicked.connect(lambda: make_pyramid_mesh(20.0))\n\
        main_layout.addWidget(btn)\n\
\n\
        self.setLayout(main_layout)\n\
        self.resize(250, 100)</code></pre>\n\
<p><strong>graphics.py</strong>:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&quot;&quot;&quot;\n\
    Provide the graphic functionality for the pryamid tool.\n\
&quot;&quot;&quot;\n\
from pymxs import runtime as rt\n\
\n\
def make_pyramid_mesh(side=20.0):\n\
    &apos;&apos;&apos;Construct a pyramid from vertices and faces.&apos;&apos;&apos;\n\
    halfside = side / 2.0\n\
    pyr = rt.mesh(\n\
        vertices=[\n\
            rt.point3(0.0, 0.0, side),\n\
            rt.point3(-halfside, -halfside, 0.0),\n\
            rt.point3(-halfside, halfside, 0.0),\n\
            rt.point3(halfside, 0.0, 0.0)\n\
            ],\n\
        faces=[\n\
            rt.point3(1, 2, 3),\n\
            rt.point3(1, 3, 4),\n\
            rt.point3(1, 4, 2),\n\
            rt.point3(2, 3, 4),\n\
        ])\n\
    rt.redrawViews()\n\
    return pyr</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../tutorials.html\">Tutorials</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";