var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"./scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"./scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"./scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_using_pymxs_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Using pymxs</h1></div><p>The <code>pymxs</code> Python module provides a wrapper for the MAXScript engine, and exposes all the interfaces, globals, and classes available in MAXScript.  This set of topics covers the basics of how to work with 3ds Max scenes and objects using <code>pymxs</code>.  Because <code>pymxs</code> is a light wrapper for MAXScript, this guide does not contain a description of every available API, but rather some overview and examples.  Virtually every API described in the MAXScript Guide can be accessed with <code>pymxs</code> (with some exceptions outlined in the <a href=\"using_pymxs/pymxs_differences.html\">Differences Between pymxs and MAXScript</a> topic).</p>\n\
<p>Note that scripts must import the <code>pymxs</code> module. </p>\n\
<p>3ds Max ships with several example scripts that illustrate how to perform various common tasks using <code>pymxs</code>.  These samples are located in <code>[3ds Max Install]/scripts/PythonSamples/</code>.</p>\n\
<h2 id=\"case-sensitivity\">Case Sensitivity</h2><p>While Python is case sensitive (so that, for example <code>print()</code> and <code>Print()</code> are not equivalent), MAXScript is not.  The convention in the MAXScript documentation is to generally camelCase functions and properties to make them easier to read, but this is not always the case in samples, or in scripts you may find elsewhere.  Because they are essentially MAXScript calls, members in <code>pymxs.runtime</code> are <strong>not</strong> case sensitive.  Both <code>pyxms.runtime.converttomesh()</code> and <code>pymxs.runtime.convertToMesh()</code> are valid, which may seem odd to someone used to Python.</p>\n\
<h2 id=\"indices\">Indices</h2><p>Another difference between Python and MAXScript is how array indices are managed.  Python is 0-based, MAXScript is 1-based.  When arrays are passed between Python and MAXScript, they are converted to the native index format.  For example, when accessing the <code>meditMaterials</code> array of material editor slots from <code>pymxs</code>, use 0-based indices:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; len(pymxs.runtime.meditmaterials)\n\
24\n\
&gt;&gt;&gt; pymxs.runtime.meditmaterials[0]\n\
&lt;Standardmaterial&lt;01 - Default:Standard&gt;&gt;</code></pre>\n\
<p>However, when using functions or methods that take index arguments, the indices are 1-based even in <code>pymxs</code>.  For example, the <code>setMeditMaterial(&lt;slot_index&gt;, &lt;material&gt;)</code> function takes as its first argument the index of a material slot.  In this case, to set the first slot:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; mat = pymxs.runtime.PhysicalMaterial()\n\
&gt;&gt;&gt; pymxs.runtime.setMeditMaterial(1, mat)</code></pre>\n\
<h2 id=\"equality-vs-identity\">Equality vs Identity</h2><p>The <code>is</code> operator is often used in Python to determine equality.  However, objects accessed through pymxs are wrapped by the pymxs layer, and though may refer to the same scene object, are not identical.  Here is an example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as mxs\n\
\n\
t = mxs.teapot(name=&apos;my teapot&apos;)\n\
t2 = mxs.getNodeByName(&apos;my teapot&apos;)\n\
print (t is t2) # this is false\n\
print (t == t2) # this is true\n\
print (f&apos;The IDs are different: t:{id(t)} t2:{id(t2)}&apos;)</code></pre>\n\
<p>As this example shows, the equality of pymxs objects should be tested using the <code>==</code> operator rather than <code>is</code>.</p>\n\
<h2 id=\"translating-maxscript-syntax-to-python\">Translating MAXScript Syntax to Python</h2><p>While the <code>pymxs</code> module is a thin wrapper for MAXScript, it is not always obvious how to translate certain MAXScript syntax into the Python world.  Here are some of the common cases:</p>\n\
<h3 id=\"keyword-arguments\">Keyword Arguments</h3><p>Many MAXScript functions take keyword parameters.  Consider this example that creates a bitmap and renders to it.  It uses two keyword parameters, <code>color:</code> when constructing the Bitmap, and <code>to:</code> when rendering.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">bm = Bitmap 320 240 color:white\n\
render to:bm\n\
display bm</code></pre><p>The equivalent in pymxs also uses keyword arguments, and looks like this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">bm = pymxs.runtime.Bitmap(320,240,color=pymxs.runtime.white)\n\
pymxs.runtime.render(to=bm)\n\
pymxs.runtime.display(bm)</code></pre>\n\
<h3 id=\"maxscript-names\">MAXScript Names</h3><p>Name literals are commonly used in MAXScript to signify options passed to functions, and have no corresponding type in Python.  They can be created with <code>pymxs.runtime.Name()</code>.  For example, to use the equivalent of <code>toolmode.coordsys #world</code>:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">pymxs.runtime.toolMode.coordsys(pymxs.runtime.Name(&quot;world&quot;))</code></pre>\n\
<p>Many functions return names or arrays of names.  For example, to get all the properties for a box, using the equivalent of <code>getpropnames box</code>:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">boxprops = pymxs.runtime.getpropnames(pymxs.runtime.box)</code></pre>\n\
<p>This yields a MAXScript Array object containing runtime.Name objects.  In this case, it might be more useful to generate a native list of Python strings:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">boxprops = [str(x) for x in list(pymxs.runtime.getpropnames(pymxs.runtime.box))]\n\
# gives: [&apos;typeinCreationMethod&apos;, &apos;typeInPos&apos;, &apos;typeInLength&apos;, &apos;typeInWidth&apos;, &apos;typeInHeight&apos;, &apos;length&apos;, &apos;width&apos;, &apos;height&apos;, &apos;widthsegs&apos;, &apos;lengthsegs&apos;, &apos;heightsegs&apos;, &apos;mapcoords&apos;]</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
      <div class=\"relinfo relchildren\">\n\
        <strong>Pages in this section</strong>\n\
        <ul class=\"ullinks\">\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_module.html\">The pymxs Module</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_objects.html\">Working with Scene Objects in pymxs</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_materials.html\">Creating and Assigning Materials</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_values.html\">MAXScript Values in pymxs</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_scenegraph.html\">Working with the Scene Graph</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_files.html\">Working With Scene Files</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_renderer.html\">Working with the Renderer</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_bitmaps.html\">Working with Bitmaps</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_animation.html\">Working with Animation</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_assets.html\">Working with Assets</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_filepaths.html\">Working with System Directories</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_registering_callbacks.html\">Registering Python Callbacks</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_plugins.html\">Working with Plug-ins</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_macroscripts_menus.html\">Working with Macroscripts and Menus</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_persistent_vars.html\">Global and Persistent Variables in pymxs</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_inspection.html\">Performing Inspection of Classes and Objects in pymxs</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_custattributes.html\">Creating Custom Attributes with pymxs</a></strong></li>\n\
          <li class=\"link ulchildlink\"><strong><a href=\"using_pymxs/pymxs_differences.html\">Differences Between pymxs and MAXScript</a></strong></li>\n\
        </ul>\n\
      </div>\n\
  \n\
    <div class=\"familylinks\">\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";