var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_using_pymxs_pymxs_differences_pymxs_by_ref_parameters_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>By-Reference Parameter Handling in pymxs</h1></div><p>Values passed back to Python from pymxs MAXScript functions that take a by-reference parameter can be translated to Python values using the <code>pymxs.byref()</code> and  <code>pymxs.mxsreference()</code> functions.  In MAXScript, &apos;by reference&apos; means that a parameter is defined and has an initial value before it is passed to a function (known as an &apos;in value&apos;), and may be modified by the function operation (known as an &apos;out value&apos;).  By reference parameters are indicated with a &apos;&amp;&apos; character. For more information about this feature, see the &quot;By Reference Parameter Passing&quot; topic in the MAXScript Help.  This behavior has no direct parallel in Python.</p>\n\
<p>The <code>pymxs.byref()</code> function is introduced in <strong>3ds Max 2021</strong>, and is the preferred way to work with by-reference parameters, as it provides a more &apos;Pythonic&apos; way of handling these parameters.</p>\n\
<h2 id=\"pymxs-byref-\">pymxs.byref()</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">pymxs.byref(PythonValue | MXSValue)</code></pre>\n\
<p>This function marks a value to be used as a by-reference parameter.  When you mark a parameter with <code>byref()</code>, the called <code>pymxs.runtime</code> function returns a tuple that contains, in order, the result of the function call, and then the updated value of each <code>byref()</code> parameter.  Values passed to <code>byref()</code> are not modified in place, as they would be in the equivalent MAXScript function call.  In other words, parameters marked with <code>byref()</code> are not updated by the function call, even if they are considered &quot;output parameters&quot;.  The updated value is contained in the returned tuple.</p>\n\
<p>If a by-reference parameter is only an &quot;output parameter&quot;, an actual value does not need to be supplied to <code>byref()</code>, the value can be passed as <code>None</code>.  If the parameter is an &quot;input parameter&quot; or &quot;input and output parameter&quot;, a value does need to be supplied.</p>\n\
<p>The values passed to <code>byref()</code> can be either Python values or MAXScript values.</p>\n\
<p>For example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import pymxs\n\
rt = pymxs.runtime\n\
\n\
b = rt.box()\n\
# call cloneNodes(), which takes a named by-reference parameter &apos;newNodes&apos;\n\
# this is only an out parameter, so we can pass it as None\n\
result, myClonedNodes = rt.maxOps.cloneNodes(b,  cloneType=rt.Name(&apos;reference&apos;), newNodes=pymxs.byref(None))\n\
print(&apos;Result: {}, clonedNodes: {}&apos;.format(result, myClonedNodes))\n\
\n\
# colorById also takes an out parameter, but it is not named:\n\
res, my_color = rt.maxOps.colorById(2, pymxs.byref(None))\n\
print(&apos;my_color: &apos;, my_color)\n\
\n\
# when dealing with an in AND out parameter, the value is returned in the tuple\n\
# NOT modified in place, as it is done in MAXScript\n\
theFileName = &apos;Simple_Stone_Mtl_Marble_bump.jpg&apos;\n\
result, theFilePath = rt.FileResolutionManager.getFullFilePath(pymxs.byref(theFileName), rt.name(&apos;bitmap&apos;))\n\
print (&apos;the input parameter was not changed: &apos;, theFileName)\n\
print (&apos;the returned output parameter is set to the full path: &apos;,  theFilePath)</code></pre>\n\
<p>The output is:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">Result: True, clonedNodes: [&lt;Box&lt;$Box:Box004 @ [0.000000,0.000000,0.000000]&gt;&gt;]\n\
my_color:  (color 255 0 0)\n\
the input parameter was not changed:  Simple_Stone_Mtl_Marble_bump.jpg\n\
the returned output parameter is set to the full path:  C:\\Program Files\\3ds Max 2021\\Maps\\adskMtl\\Simple_Stone_Mtl_Marble_bump.jpg</code></pre><h3 id=\"pymxs-mxsreference-em-deprecated-em-\">pymxs.mxsreference() <em>Deprecated</em></h3><pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">pymxs.mxsreference(PythonList | MXSValue)</code></pre>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> As of <strong>3ds Max 2021</strong>, this function is marked as deprecated, and will produce a warning in the Listener window when called the first time.  Use <code>pymxs.byref()</code> instead.</p>\n\
\n\
</div></div>\n\
\n\
<p>If the parameter is an &quot;out parameter&quot; (meaning the function updates its value), it will be updated if:</p>\n\
<ul>\n\
<li>it is a Python list</li>\n\
<li>it is a MXSValue that is not auto-converted to a Python value</li>\n\
</ul>\n\
<p>It will not be updated if:</p>\n\
<ul>\n\
<li>it is a Python value (other than list)</li>\n\
<li>it is a MXSValue that is auto-converted to a Python value</li>\n\
</ul>\n\
<p>The following MAXScript types are auto-converted to Python types: string, int, int64, intptr, float, double, boolean, objectset, collection (but not bitarray or MXSModifierArray).  </p>\n\
<p>For example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import pymxs\n\
from pymxs import runtime as rt\n\
\n\
my_sphere = rt.sphere(radius=10)\n\
my_type = rt.name(&quot;reference&quot;)\n\
\n\
offset1 = rt.Point3(50.0, 50.0, 10.0)\n\
# create a Python list to update with the new nodes:\n\
my_returned_list = []\n\
# pass it using mxsference() so it can be updated:\n\
rt.maxOps.cloneNodes(my_sphere,  offset=offset1, cloneType=my_type, newNodes=pymxs.mxsreference(my_returned_list))\n\
print(&quot;python array: {}&quot;.format(my_returned_list))\n\
\n\
# we can also use a MAXScript Array object to pass:\n\
\n\
rt.mra = rt.array()\n\
my_result_array = rt.mra\n\
offset2 = rt.Point3(-50.0, -50.0, 10.0)\n\
rt.maxOps.cloneNodes(my_sphere,  offset=offset2, cloneType=my_type, newNodes=pymxs.mxsreference(my_result_array))\n\
print(&quot;runtime array: {}&quot;.format(my_result_array))</code></pre>\n\
<p>The result printed to the Listener is:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">python array: \n\
[&lt;Sphere&lt;$Sphere:Sphere005 @ [50.000000,50.000000,10.000000]&gt;&gt;]\n\
runtime array: \n\
#($Sphere:Sphere006 @ [-50.000000,-50.000000,10.000000])</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../pymxs_differences.html\">Differences Between pymxs and MAXScript</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";