var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"..\\../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"..\\../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_using_pymxs_pymxs_differences_pymxs_controllers_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/Max-Python-API/scripts/prettify/lang-mxs.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Accessing Object Properties and Controllers</h1></div><p>Working with nested properties, such as controllers, in pymxs requires a different approach than in MAXScript.  MAXScript &quot;knows&quot; about nested object properties, and does some extra parsing when it encounters them.  Python and MAXScript treat properties indicated with dot notation differently: Python always returns a reference to the indicated property, while MAXScript returns a copy of the value.  Attempting to assign a value may not work as intended in pymxs.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<div class=\"note note:\"><strong>Note</strong>: There are some exceptions to this situation, as when the underlying property is a reference.  So something like <code>box.controller.position</code> will work, because controller is a reference.</div>\n\
\n\
</div></div>\n\
\n\
<p>For example, in MAXScript you can do this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">b = box()\n\
b.pos.x = 20</code></pre><p>The equivalent in pymxs does not work as intended, because <code>b.pos.x</code> is returning a copy of the pos object, which is no longer connected to the box&apos;s position.</p>\n\
<p>There are three solutions to this problem:</p>\n\
<ol>\n\
<li>Use the MAXScript <code>getProperty()</code> and <code>setProperty()</code> functions</li>\n\
<li>Use the pymxs MXSWrapperBase <code>getmxsprop()</code> and <code>setmxsprop()</code> functions</li>\n\
<li>Work on a copy of the target property, and assign the object back</li>\n\
</ol>\n\
<p>For example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as mxs\n\
b = mxs.box()\n\
\n\
b.pos.x = 9  # trying to set the x value of the box position, but we got a copy of the position\n\
print(b.pos.x)  # see, we are still at pos.x == 0\n\
\n\
# solution: use MXSWrapper.setmxsprop()\n\
b.setmxsprop(&quot;pos.x&quot;, 9.0)  # properly set the nested property value\n\
print(b.pos.x)  # the x value of the box position is now at 9\n\
\n\
\n\
# OR use MAXScript setProperty()\n\
mxs.setProperty(b, &quot;pos.x&quot;, 10.0)\n\
print(mxs.getProperty(b, &quot;pos.x&quot;))\n\
\n\
# you can also do this:\n\
b_pos = b.pos\n\
b_pos.x = 10  # set the copied value from the box property\n\
b.pos = b_pos  # properly apply back the changed pos value to the box\n\
print(b.pos.x)  # the x value of the box position is now at 10\n\
\n\
# getting the position:\n\
print(b.pos.x)  # 10.0 - fetches the proper value, but it is a copy of the position\n\
print(b.getmxsprop(&quot;pos.x&quot;)) # 10.0 - fetches the proper value\n\
print(mxs.getProperty(b, &quot;pos.x&quot;)) # 10.0 - also works</code></pre>\n\
<h2 id=\"working-with-controllers\">Working with Controllers</h2><p>Here is an example of some MAXScript code:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-mxs prettyprint\">myTeapot = teapot()\n\
myTeapot.pos.controller = noise_position()</code></pre>\n\
<p>In MAXScript, as in Python, <code>myTeapot.pos</code> is a Point3, which doesn&apos;t have an associated controller property.  Attempting to get <code>myTeapot.pos.controller</code> in Python results in an error such as: <code>AttributeError: &apos;pymxs.MXSWrapperBase&apos; object has no attribute &apos;controller&apos;</code>.</p>\n\
<p>But in MAXScript <code>myTeapot.pos.controller</code> is perfectly valid, as it evaluates to an object structure hierarchy. Therefore, to achieve the same thing using pymxs, we must use a different approach and use <code>getPropertyController()</code> / <code>setPropertyController()</code>.</p>\n\
<p>Here&apos;s a complete example where we set the Position XYZ controller of a box, and the X component of the position for a sphere:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as rt\n\
\n\
# set the transform -&gt; position controller\n\
t = rt.teapot()\n\
rt.setPropertyController( t.controller, &apos;Position&apos;, rt.noise_position())\n\
print(rt.getPropertyController(t.controller, &apos;Position&apos;))\n\
\n\
# set the transform -&gt; position -&gt; x_position controller\n\
s = rt.sphere()\n\
sphr_pos_ctrl = rt.getPropertyController(s.controller, &apos;Position&apos;)\n\
rt.setPropertyController(sphr_pos_ctrl, &apos;X Position&apos;, rt.noise_float())\n\
print(rt.getPropertyController(sphr_pos_ctrl, &apos;X Position&apos;))</code></pre>\n\
<p>Output:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">Controller:Noise_Position\n\
Controller:Noise_Float</code></pre>\n\
<h2 id=\"expression-controllers\">Expression Controllers</h2><p>Expression controllers are created and set in the same way, though they require the extra steps of adding an expression and (if required) scalar and vector targets.  In this example, we add an expression controller to the X Position sub-controller of an object&apos;s position controller, so that its X position always matches a target object.</p>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> The name of the controller you pass to getPropertyController() is the same string that appears in the <strong>Track View - Curve Editor</strong> in the 3ds Max UI.</div>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as rt\n\
\n\
# target:\n\
s = rt.sphere()\n\
\n\
t = rt.teapot(pos=rt.point3(20,20,0))\n\
s_pos_ctl = rt.getPropertyController(s.controller, &apos;Position&apos;)\n\
expr_pos_ctl = rt.position_expression()\n\
# first add the scalar variable targeting the sphere&apos;s X position\n\
expr_pos_ctl.addScalarTarget(&apos;sphere_x&apos;, rt.getPropertyController(s_pos_ctl, &apos;X Position&apos;))\n\
# then add the expression that uses that variable\n\
expr_pos_ctl.setExpression(&apos;[sphere_x, 20,0]&apos;)\n\
# finally, set the controller\n\
rt.setPropertyController(t.controller, &quot;Position&quot;, expr_pos_ctl)\n\
</code></pre>\n\
<h2 id=\"assigning-controllers-where-there-is-not-an-existing-controller\">Assigning Controllers Where There is not an Existing Controller</h2><p>The transform properties of an object has default controllers, and can be accessed and modified as explained above.  Adding a controller to a property that does not currently have one with pymxs is not as straightforward, because there is no name associated with the property&apos;s controller. </p>\n\
<p>3ds Max stores all the controllers for properties for an object in a structure called a <em>subanim</em> (sub-animatable).  A subanim can contain nested subanims.  To assign a controller to a property that does not have a controller, we need to find the property&apos;s location in the subanim tree structure by index, and assign the controller to the result.  We can do this using the MAXScript function <code>getSubanim()</code>.</p>\n\
<p>For example, to assign a float controller to an object&apos;s <strong>width</strong> property:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as mxs\n\
b = mxs.box()\n\
bw= mxs.getSubAnim(mxs.getSubAnim(b, 4), 2)\n\
mxs.setProperty(bw, &apos;controller&apos;, mxs.bezier_float())</code></pre>\n\
<p>This gets the box node&apos;s fourth subAnim (the box object), and then gets the second subAnim (the width property).</p>\n\
<p>Note that the <code>getSubAnim()</code> function gets the subAnim, while <code>getPropertyController()</code> (illustrated above) gets the controller for the subAnim.  So to expand the previous example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as mxs\n\
b = mxs.box()\n\
# get the subAnim for X_Rotation:\n\
brx = mxs.getSubAnim(mxs.getSubAnim(mxs.getSubAnim(b, 3), 2), 1) \n\
# get the controller for X_Rotation:\n\
brxc = mxs.getPropertyController(mxs.getPropertyController(b.controller, &apos;Rotation&apos;), &apos;X Rotation&apos;)\n\
brx.controller == brxc # this is True</code></pre>\n\
<p>This example illustrates visiting all the subAnims in an object&apos;s subAnim tree, and prints out the corresponding names and indices.  It can be used to determine the correct indices to use to assign new controllers:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as mxs\n\
\n\
b = mxs.box()\n\
\n\
for sub_i in range(b.numsubs):\n\
    sub = mxs.getSubAnim(b, sub_i+1)\n\
    print(f&apos;{sub.name} [{sub_i+1}]&apos;)\n\
\n\
    if sub.controller == None:\n\
        print(&apos; No controller assigned to it&apos;)\n\
    else:\n\
        print(f&apos; current controller: {(sub.controller)}&apos;)\n\
    if sub.keys == None:\n\
        print(&apos; No keys assigned to it&apos;)\n\
\n\
    for secsub_i in range(sub.numsubs):\n\
        secsub = mxs.getSubAnim(sub, secsub_i+1)\n\
        print(f&apos;\\t{secsub.name} [{secsub_i+1}]&apos;)\n\
\n\
        if secsub.controller == None:\n\
            print(&apos;\\t No controller assigned to it&apos;)\n\
        else:\n\
            print(f&apos;\\t current controller: {(secsub.controller)}&apos;)\n\
        if secsub.keys == None:\n\
            print(&apos;\\t No keys assigned to it&apos;)\n\
        for thirdsub_i in range(secsub.numsubs):\n\
            thirdsub = mxs.getSubAnim(secsub, thirdsub_i+1)\n\
            print(f&apos;\\t\\t{thirdsub.name} [{thirdsub_i+1}]&apos;)\n\
            if secsub.controller == None:\n\
                print(&apos;\\t No controller assigned to it&apos;)\n\
            else:\n\
                print(f&apos;\\t\\t current controller: {(thirdsub.controller)}&apos;)</code></pre>\n\
<p>This displays the following output:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">Visibility [1]\n\
 No controller assigned to it\n\
 No keys assigned to it\n\
Space Warps [2]\n\
 No controller assigned to it\n\
 No keys assigned to it\n\
Transform [3]\n\
 current controller: Controller:Position_Rotation_Scale\n\
    Position [1]\n\
     current controller: Controller:Position_XYZ\n\
        X Position [1]\n\
         current controller: Controller:Bezier_Float\n\
        Y Position [2]\n\
         current controller: Controller:Bezier_Float\n\
        Z Position [3]\n\
         current controller: Controller:Bezier_Float\n\
    Rotation [2]\n\
     current controller: Controller:Euler_XYZ\n\
        X Rotation [1]\n\
         current controller: Controller:Bezier_Float\n\
        Y Rotation [2]\n\
         current controller: Controller:Bezier_Float\n\
        Z Rotation [3]\n\
         current controller: Controller:Bezier_Float\n\
    Scale [3]\n\
     current controller: Controller:Bezier_Scale\n\
Object (Box) [4]\n\
 No controller assigned to it\n\
 No keys assigned to it\n\
    Length [1]\n\
     No controller assigned to it\n\
     No keys assigned to it\n\
    Width [2]\n\
     No controller assigned to it\n\
     No keys assigned to it\n\
    Height [3]\n\
     No controller assigned to it\n\
     No keys assigned to it\n\
    Width Segments [4]\n\
     No controller assigned to it\n\
     No keys assigned to it\n\
    Length Segments [5]\n\
     No controller assigned to it\n\
     No keys assigned to it\n\
    Height Segments [6]\n\
     No controller assigned to it\n\
     No keys assigned to it\n\
Material [5]\n\
 No controller assigned to it\n\
 No keys assigned to it\n\
Image Motion Blur Multiplier [6]\n\
 No controller assigned to it\n\
 No keys assigned to it\n\
Object Motion Blur On Off [7]\n\
 No controller assigned to it\n\
 No keys assigned to it</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../pymxs_differences.html\">Differences Between pymxs and MAXScript</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";