var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_using_pymxs_pymxs_module_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/Max-Python-API/scripts/prettify/lang-mxs.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>The <code>pymxs</code> Module</h1></div><p>The pymxs module provides access to the MAXScript runtime, as well as some top-level functions and types.  This topic covers those top-level members:</p>\n\
<ul>\n\
<li>The MAXScript runtime, <code>pymxs.runtime</code></li>\n\
<li>Context expression functions, such as <code>pymxs.animate</code></li>\n\
<li>The MAXScript undo system functions, <code>run_undo()</code> and <code>run_redo()</code></li>\n\
<li>The <code>pymxs.print_()</code> function</li>\n\
<li>The <code>pymxs.byref()</code> function, to handle those situations where a MAXScript function takes a by-reference parameter</li>\n\
<li>The <code>pymxs.mxstoken()</code> function, to handle access to pymxs.runtime in multi-threaded situations</li>\n\
</ul>\n\
<!-- TOC -->\n\
\n\
<ul>\n\
<li><a href=\"#the-maxscript-runtime\">The MAXScript Runtime</a></li>\n\
<li><a href=\"#context-expression-functions\">Context Expression Functions</a></li>\n\
<li><a href=\"#maxscript-undo-and-redo-with-pymxsrun_undo-and-pymxsrun_redo\">MAXScript Undo and Redo with <code>pymxs.run_undo()</code> and <code>pymxs.run_redo()</code></a></li>\n\
<li><a href=\"#printing-to-the-listener-with-pymxsprint_\">Printing to the Listener with pymxs.print_()</a></li>\n\
<li><a href=\"#by-reference-parameter-handling\">By-Reference Parameter Handling</a></li>\n\
<li><a href=\"#accessing-the-runtime-in-worker-threads-with-pymxsmxstoken\">Accessing the Runtime In Worker Threads with <code>pymxs.mxstoken()</code></a></li>\n\
</ul>\n\
<!-- /TOC -->\n\
\n\
<h2 id=\"the-maxscript-runtime\">The MAXScript Runtime</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">pymxs.runtime</code></pre>\n\
<p>This member exposes the MAXScript runtime, and all the global symbols it contains, to Python scripts. The runtime is dynamic, so global objects created in MAXScript are subsequently available to <code>pymxs.runtime</code>.</p>\n\
<p>The names of objects available in the <code>pymxs.runtime</code> module correspond to MAXScript globals. So, for example, to create a teapot you can use the MAXScript <code>Teapot()</code> constructor like this:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; import pymxs\n\
&gt;&gt;&gt; rt = pymxs.runtime\n\
&gt;&gt;&gt; t = rt.Teapot()\n\
&gt;&gt;&gt; rt.classOf(t)\n\
&lt;GeometryClass&lt;Teapot&gt;&gt;</code></pre>\n\
<h2 id=\"context-expression-functions\">Context Expression Functions</h2><p>Several MAXScript &quot;context expressions&quot; are exposed to pymxs via top-level functions.  These members are:</p>\n\
<ul>\n\
<li><code>pymxs.animate(True|False)</code></li>\n\
<li><code>pymxs.attime(True|False)</code></li>\n\
<li><code>pymxs.atlevel(True|False)</code></li>\n\
<li><code>pymxs.quiet(True|False)</code></li>\n\
<li><code>pymxs.redraw(True|False)</code></li>\n\
<li><code>pymxs.undo(True|False)</code></li>\n\
</ul>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> the following MAXScript context expressions are not available in pymxs: <code>coordsys</code>, <code>about</code>, <code>printAllElements</code>, <code>defaultAction</code>, <code>MXSCallstackCaptureEnabled</code>, <code>dontRepeatMessages</code>, and <code>macroRecorderEmitterEnabled</code>.  For <code>coordsys</code>, you can achieve the same effect by changing the current coordinate system using <code>pymxs.runtime.setRefCoordSys(mode_name)</code>.</p>\n\
\n\
</div></div>\n\
\n\
<p>In MAXScript, a context expression is used in a scoped code block, for example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-mxs prettyprint\">t = teapot()\n\
with animate on\n\
(\n\
  at time 0 t.position = [-100, 0, 0]\n\
  at time 100 t.position = [100, 0, 0]\n\
)</code></pre>\n\
<p>Here, the <code>with animate</code> context applies to the code block, while the <code>at time</code> context applies to each statement.  Note that the <code>with</code> is optional in MAXScript, and you often do not see it used.</p>\n\
<p>In pymxs we would do something similar by scoping using the Python <code>with</code> statement:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import pymxs\n\
rt = pymxs.runtime\n\
t = rt.Teapot()\n\
\n\
with pymxs.animate(True):\n\
    with pymxs.attime(0):\n\
            t.pos = rt.Point3(-100, 0, 0)\n\
    with pymxs.attime(100):\n\
            t.pos = rt.Point3(100, 0, 0)</code></pre>\n\
<h2 id=\"maxscript-undo-and-redo-with-code-pymxs-run_undo-code-and-code-pymxs-run_redo-code-\">MAXScript Undo and Redo with <code>pymxs.run_undo()</code> and <code>pymxs.run_redo()</code></h2><ul>\n\
<li><code>pymxs.undo(True|False)</code></li>\n\
<li><code>pymxs.run_undo()</code></li>\n\
<li><code>pymxs.run_redo()</code></li>\n\
</ul>\n\
<p>Most 3ds Max Commands (those invoked by <code>max ...</code>) are not directly available in pymxs.runtime (see the pymxs Limitations and Workarounds topic).  However, the two functions <code>pymxs.run_undo()</code> and <code>pymxs.run_redo()</code> are mapped to <code>max undo</code> and <code>max redo</code> respectively to provide direct access to these commands.</p>\n\
<p>As with MAXScript, operations run through pymxs are not recorded by the\n\
undo system.  Undo must be specifically enabled.  Undoable pymxs operations\n\
appear as items labeled <em>MAXScript</em> in the 3ds Max Undo menu.</p>\n\
<p>For example, to create a teapot, change its position, and then undo the change:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import pymxs\n\
rt = pymxs.runtime\n\
# this will not be undoable:\n\
with pymxs.undo(False):\n\
    t = rt.Teapot()\n\
\n\
with pymxs.undo(True):\n\
    t.pos = rt.Point3(20,20,20)\n\
\n\
pymxs.run_undo() # undo the position assignment to [20,20,20]\n\
pymxs.run_redo() # redo the position assignement to [20,20,20]</code></pre>\n\
<h3 id=\"undo-redo-and-exceptions\">Undo / Redo and Exceptions</h3><p>The way an undo block in pymxs handles exceptions is different than in MAXScript.  When an exception is raised, code executed before the exception is undone.  The exception is not raised beyond the scope of the undo block, as the undo operation is considered to be the exception handler.  Here is an example to illustrate this behavior:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\"># undo with exception\n\
\n\
import pymxs\n\
from pymxs import runtime as rt\n\
\n\
def make_box_and_raise():\n\
    with pymxs.undo(True, &apos;Making box&apos;):\n\
        rt.Box()\n\
        # this will undo the box, and the error will not show up\n\
        # in the listener, because it is handled:\n\
        raise RuntimeError(&apos;This is an error&apos;)\n\
\n\
def make_sphere_and_raise():\n\
    rt.Sphere()\n\
    # this will show up in the lister as an unhandled exception\n\
    raise RuntimeError(&apos;This is an error&apos;)\n\
\n\
make_box_and_raise()\n\
make_sphere_and_raise()</code></pre>\n\
<h2 id=\"printing-to-the-listener-with-pymxs-print_-\">Printing to the Listener with pymxs.print_()</h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">pymxs.print_(msg[, isErr[, [forceOnMainThread]])</code></pre>\n\
<p>This member gives you more control over error formatting and threading behavior.  </p>\n\
<ul>\n\
<li><code>msg</code> - the string to print. </li>\n\
<li><code>isErr</code> - an optional boolean that indicates whether the message should be formatted as an error (in red, by default).  The default is False.</li>\n\
<li><code>forceOnMainThread</code> - an optional boolean that indicates whether the print should happen on the main thread.  In a threaded script, setting this to True ensures that output is sent to the Listener window even if called from a worker thread.  If False, when called from a worker thread, output is sent to stdout/stderr (the <em>3dsmax_ouput.log</em> file or console).  The default is True.</li>\n\
</ul>\n\
<p>This function exists because both MAXScript <code>print()</code> and <code>assert()</code> functions are disabled and not callable from <code>pymxs.runtime</code>.  There are a couple of ways to work around this limitation to send messages to the Listener window.  The simplest is to use the standard Python <code>print()</code> function and <code>assert</code> statement.  However, you cannot control formatting or threading behavior with these functions, so there may be situations where <code>pymxs.print_()</code> is the better solution.</p>\n\
<h2 id=\"by-reference-parameter-handling\">By-Reference Parameter Handling</h2><p>The <code>pymxs.byref()</code> function allows you to use MAXScript functions that take &quot;by reference&quot; parameters. See the <a href=\"pymxs_differences/pymxs_by_ref_parameters.html\">By-Reference Parameter Handling in pymxs</a> topic for more information.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> As of <strong>3ds Max 2021</strong>, the <code>pymxs.mxsreference()</code> function is marked as deprecated, and will produce a warning in the Listener window when called the first time.  Use <code>pymxs.byref()</code> instead.</p>\n\
\n\
</div></div>\n\
\n\
<h2 id=\"accessing-the-runtime-in-worker-threads-with-code-pymxs-mxstoken-code-\">Accessing the Runtime In Worker Threads with <code>pymxs.mxstoken()</code></h2><pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">pymxs.mxstoken()</code></pre>\n\
<p>You cannot call  <em>pymxs.runtime</em>  commands directly on a worker thread, as this will cause a runtime exception. However,  <em>pymxs</em>  has a <code>mxstoken()</code> function to support multi-threading by providing a synchronization mechanism between worker threads.</p>\n\
<p>For an example of this approach, see the example <strong>\\&lt;maxdirectory&gt;\\scripts\\Python\\demoMXSToken.py</strong>.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../using_pymxs.html\">Using pymxs</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";