var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_using_pymxs_pymxs_objects_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Working with Scene Objects in <code>pymxs</code></h1></div><p>This topic is an introduction to creating and manipulating scene objects in 3ds Max using pymxs.  This is an overview, and contains some examples.  For more comprehensive information about all objects you can create and manipulate, see the <strong>Node: MAXWrapper</strong> group of topics (under <strong>3ds Max Objects and Interfaces</strong>) in the <em>MAXScript Help</em>.</p>\n\
<p>Objects in a 3ds Max scene include geometry, shapes, lights, cameras, helpers, spacewarps, and systems. They are organized in a tree structure called the <em>scene graph</em>, with the help of a node object, with a root node at the top. Each node can have one parent and multiple children. The Scene Explorer reflects this structure. Each scene graph node stores information about the object at that location in the scene graph, such as its name, position rotation and scale, shading, modifiers, visibility, layers, and so on. For more information, see <a href=\"pymxs_scenegraph.html\">Working with the Scene Graph</a>.</p>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<p><strong></strong> Most examples in this topic are run from the MAXScript Listener window, which also has a Python commandline mode.  This allows us to see results immediately in 3ds Max, and to query objects.  Examples run in the listener are indicated with the Listener Python prompt <code>&gt;&gt;&gt;</code>.</p>\n\
\n\
</div></div>\n\
\n\
<h2 id=\"creating-a-scene-object\">Creating a Scene Object</h2><p>Objects are created with constructors.  All of the properties of an object can optionally be set when it is constructed, and if they are omitted the object&apos;s defaults are used.  For example, to create a teapot:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; my_teapot = pymxs.runtime.teapot()\n\
&gt;&gt;&gt; my_teapot\n\
&lt;Teapot&lt;$Teapot:Teapot001 @ [0.000000,0.000000,0.000000]&gt;&gt;</code></pre>\n\
<p>This creates a new teapot object at position [0,0,0], with a radius of 25 and 4 segments.  To change these parameters at creation time, we can specify them in the constructor:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; teapot_position=pymxs.runtime.point3(100,20,10)\n\
&gt;&gt;&gt; my_teapot = pymxs.runtime.teapot(radius=50, pos=teapot_position, segments=2)</code></pre>\n\
<p>We can now manipulate any of the object&apos;s properties.  For example, to turn our teapot into a simple pot by removing its handle and spout:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; my_teapot.spout = False\n\
&gt;&gt;&gt; my_teapot.handle = False</code></pre>\n\
<p>All of the available properties for a teapot are documented in the <strong>Teapot: GeometryClass</strong> topic in the <em>MAXScript Help</em>.</p>\n\
<p>We can also use any of the common properties, operators or methods exposed by the <code>Node</code> class.  For example, we can query whether the object is a Shape (with <code>isShapeObject()</code>) and get the number of face and vertices (with <code>getPolygonCount()</code>):</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; pymxs.runtime.isShapeObject(my_teapot)\n\
False\n\
&gt;&gt;&gt; pymxs.runtime.getPolygonCount(my_teapot)\n\
&lt;Array&lt;#(192, 98)&gt;&gt;</code></pre>\n\
<h2 id=\"modifying-the-object\">Modifying the Object</h2><p>A modifier transforms an object.  Most modifiers are &quot;object modifiers&quot; that are applied to objects and transform them in their own local space, and perform deformations such as bend, taper and twist.</p>\n\
<p>A modifier takes an object and either modifies it directly, or creates a new object from the input object. Many modifiers only apply to certain characteristics (that is, channels) of an object such as the vertices or the faces.</p>\n\
<p>Modifiers can be specific to certain types of object, for example, the Volume Select Modifier only works on triangle mesh objects (TriObject).</p>\n\
<p>To modify our object from the previous example, we first create a new modifier, then set it&apos;s properties, and then add it to the object (though you can also add it to the object, and then set the properties).</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; my_modifier = pymxs.runtime.taper()\n\
&gt;&gt;&gt; my_modifier.amount = 2.0\n\
&gt;&gt;&gt; my_modifier.curve = 1.5\n\
&gt;&gt;&gt; pymxs.runtime.addmodifier(my_teapot, my_modifier)</code></pre>\n\
<h2 id=\"editing-a-mesh\">Editing a Mesh</h2><p>There are two ways to work with a geometry object&apos;s mesh in pymxs/MAXScript: convert the object to a mesh object (such as Editable_Poly) using <code>pymxs.runtime.convertToMesh()</code> or <code>convertToPoly()</code>, or apply one of the mesh modifiers to the object (such as Edit_Poly).  Collapsing an object with modifers applied to it also converts it to a mesh object, an Editable_Mesh (a tri-mesh) by default, or an Editable_Poly if Edit_Poly is in the modifier stack.</p>\n\
<p>It is also possible to construct a mesh from scratch in pymxs/MAXScript.  See the <code>mesh_and_cpv.py</code> sample for an example of how you could do this.</p>\n\
<h2 id=\"transforming-scale-rotation-and-position\">Transforming: Scale, Rotation and Position</h2><p>There are three transforms related to scene nodes: the Node Transform Matrix (held in the object&apos;s <code>.transform</code> property), the Object-Offset Transform Matrix (held in the object&apos;s <code>.objectoffsetpos</code>, <code>.objectoffsetrot</code>, and <code>.objectoffsetscale</code> properties), and the Object Transform Matrix, which is the combination of the Node Transform and Object-Offset matrices.  This is held in the object&apos;s <code>.objecttransform</code> property.</p>\n\
<p>For a detailed discussion of how to access and manipulate these transforms, as well as the &quot;shortcut&quot; properties such as .pos and .rotation, see the &quot;Using Node Transform Properties&quot; topic in the <em>MAXScript Help</em>.</p>\n\
<p>This example script is an adapation of the sample script in that topic adapted to use pymxs:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as rt\n\
\n\
# define function for coercing a rotation to a quat\n\
rt.execute(&quot;fn r2q r = (return r as quat)&quot;)\n\
\n\
def DumpXForms (obj):\n\
    # output node transform properties\n\
    print &quot;{}:\\t{}&quot;.format(&quot;transform&quot;, obj.transform)\n\
    print &quot;{}:\\t{}&quot;.format(&quot;position &quot;, obj.pos)\n\
    print &quot;{}:\\t{}&quot;.format(&quot;rotation &quot;, obj.rotation)\n\
    # output node&apos;s pivot point location\n\
    print &quot;{}:\\t{}&quot;.format(&quot;pivot &quot;, obj.pivot)\n\
    # output object offsets\n\
    print &quot;{}:\\t{}&quot;.format(&quot;objectoffsetpos&quot;, obj.objectoffsetpos)\n\
    print &quot;{}:\\t{}&quot;.format(&quot;objectoffsetrot&quot;, obj.objectoffsetrot)\n\
    print &quot;{}:\\t{}&quot;.format(&quot;objectoffsetscale&quot;, obj.objectoffsetscale)\n\
    # output object transform\n\
    print &quot;{}:\\t{}&quot;.format(&quot;objecttransform &quot;, obj.objecttransform)\n\
    # output vertex position in local and world coordinates\n\
    # we do this because &apos;in coordsys&apos; is not available in pymxs\n\
    rt.setRefCoordSys(rt.Name(&apos;local&apos;))\n\
    print &quot;{}:\\t{}&quot;.format(&quot;vert 1 (local) &quot;, (rt.getvert( obj, 1)))\n\
    rt.setRefCoordSys(rt.Name(&apos;world&apos;))\n\
    print &quot;{}:\\t{}&quot;.format(&quot;vert 1 (world1) &quot;, (rt.getvert( obj, 1)))\n\
    # calculate and output vertex position in world coordinates\n\
    v_pos = rt.getvert( obj, 1) * obj.objecttransform\n\
    print&quot;{}:\\t{}&quot;.format(&quot;vert 1 (world2) &quot;, v_pos)\n\
\n\
\n\
\n\
# define function for rotating only the pivot point\n\
def RotatePivotOnly( obj, rotation):\n\
    rotValInv=rt.inverse (rt.r2q(rotation))\n\
    with pymxs.animate(False):\n\
        obj.rotation*=rotValInv\n\
        obj.objectoffsetpos*=rotValInv\n\
        obj.objectoffsetrot*=rotValInv\n\
\n\
\n\
b=rt.box(pos=rt.Point3(75,75,0)) # create a 25x25x25 box, vertex 1 at [62.5,62.5,0] (world)\n\
rt.convertToMesh (b) # convert box to mesh so we can access the vertex location\n\
DumpXForms(b) # print transforms\n\
b.pivot= rt.Point3(50,50,0)# move pivot only to [50,50,0]\n\
DumpXForms (b) # print transforms\n\
rotation = rt.EulerAngles( 0, 0, 35)\n\
RotatePivotOnly (b, rotation) # rotate pivot only 35 degrees about local Z\n\
DumpXForms (b)# print transforms</code></pre>\n\
<h2 id=\"getting-custom-attributes\">Getting Custom Attributes</h2><p>You can access custom attributes on an object by using <code>pymxs.runtime.custattributes</code>:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as rt\n\
print rt.custattributes.getpblockdefs(rt.custattributes.getdef(rt.getnodebyname(&quot;Sphere001&quot;), 1))</code></pre>\n\
<p>Assuming $Sphere001 has a custom float attribute, the result would be:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">#(#(#main, 0, 0, #(#rollout, #params), #(#myattr, #(#type, #float, #ui, #(#myattr), #default, 0.0))))</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../using_pymxs.html\">Using pymxs</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";