var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_using_pymxs_pymxs_persistent_vars_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Global and Persistent Variables in pymxs</h1></div><p>Global variables in MAXScript are &quot;top level&quot; variables that have global scope. They can be explicitly declared (using the <code>global</code> keyword), or implicitly declared outside of any scope.  See the &quot;Scope of Variables&quot; topic in the MAXScript Help.</p>\n\
<p>Persistent variables are global variables that are attached to a scene and are retained on scene close and open, and between 3ds Max sessions.  They are created either by using the <code>persistent</code> keyword when declaring the variable, or by using <code>peristents.make()</code> on the name of an existing global variable.  See the &quot;Persistent Global Variables&quot; topic in the MAXScript Help.</p>\n\
<p>In this example, we create a few global variables, and then make them persistent.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\"># Globals and Persistents\n\
\n\
from pymxs import runtime as rt\n\
\n\
# As of 3ds Max 2021, we can implictly create variables from Python:\n\
rt.a = &apos;hello&apos;\n\
\n\
# Pre-2021, you need to use pymxs.runtime.execute\n\
# this is one way to explicitly declare a global variable:\n\
rt.execute(&apos;global b = &quot;hello again&quot;&apos;)\n\
\n\
# Any variable declared outside of a scope is implictly considered global,\n\
# so this also works:\n\
rt.execute(&apos;c = &quot;goodbye&quot;&apos;)\n\
\n\
# To view all globals currently declared:\n\
globals = rt.globalVars.gather()\n\
print (globals)\n\
\n\
# getting and setting values requires a MAXScript Name \n\
val = rt.globalVars.get(rt.name(&apos;a&apos;))\n\
print (&apos;a={}&apos;.format(val))\n\
\n\
# Persistent variables can be created on declaration, or made persistent later on\n\
# Re-declare a:\n\
rt.execute(&apos;persistent global a = &quot;hello&quot;&apos;)\n\
\n\
# persistents.make() takes a name value of an existing global variable, and\n\
# is preferable because it does not require pymxs.runtime.execute():\n\
rt.persistents.make(rt.name(&apos;b&apos;)) \n\
\n\
# We can get all the global persistent variables with persistents.gather\n\
val = rt.persistents.gather()\n\
print (&apos;gather={}&apos;.format(val))</code></pre>\n\
<p>The output is:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">#(#NodeColorPicker, #getModContextBBoxMax, #Summed_area, #MAXCurveCtl, #setSplitMesh, #arnold_spherical_camera, #joinSurfaces, #StereoCameraRig, #log, #ai_fraction, #PxStepSimulation, #renderers, #PFBoxMeshWrapper, #Light__Area, #fetchMaxFile, #getSplitMesh, #Color_RGB, #Brightness_and_Contrast, #MSPluginClass, #g_ep, ...)\n\
a=hello\n\
gather=#(#b, #CAT_UINum, #a)</code></pre>\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../using_pymxs.html\">Using pymxs</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";