var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_using_pymxs_pymxs_renderer_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Working with the Renderer</h1></div><p>This topic discusses ways to perform rendering tasks using pymxs.</p>\n\
<p><a name=\"toc\"></a></p>\n\
<!-- toc -->\n\
\n\
<ul>\n\
<li><a href=\"#executing-your-render\">Executing your render</a></li>\n\
<li><a href=\"#rendering-to-an-output-file\">Rendering to an output file</a></li>\n\
<li><a href=\"#using-pre-and-post-render-scripts\">Using pre- and post-render scripts</a></li>\n\
<li><a href=\"#rendering-settings-and-dialogs\">Rendering Settings and Dialogs</a></li>\n\
<li><a href=\"#working-with-the-viewport\">Working with the viewport</a><ul>\n\
<li><a href=\"#iterating-through-the-list-of-viewports\">Iterating through the list of viewports</a></li>\n\
<li><a href=\"#maximizing-and-zooming-a-viewport\">Maximizing and zooming a viewport</a></li>\n\
<li><a href=\"#saving-the-active-viewport-graphics-window-as-a-bitmap\">Saving the active viewport graphics window as a bitmap</a></li>\n\
</ul>\n\
</li>\n\
</ul>\n\
<!-- tocstop -->\n\
\n\
<h2 id=\"executing-your-render\">Executing your render</h2><p>The <code>render()</code> function invokes the 3ds Max renderer, and takes many optional parameters to control the rendering independent of the current <strong>Render Settings Dialog</strong> settings.</p>\n\
<p>By default this function returns an in-memory render as a bitmap value.</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\"># this will display the render in a virtual file buffer window\n\
bmp = pymxs.runtime.render(pos=pymxs.runtime.name(&apos;vfb_upper_left&apos;))\n\
# the bitmap can also be displayed later:\n\
pymxs.runtime.display(bmp, caption=&apos;My Bitmap&apos;)</code></pre>\n\
<h2 id=\"rendering-to-an-output-file\">Rendering to an output file</h2><p>The <code>render()</code> function can save the render to a file if the <code>outputFile:</code> parameter is specified.</p>\n\
<p>For example, this function (from the <code>render.py</code> pymxs sample) saves the rendered output as a single-frame jpeg file to the system default <code>#renderoutput</code> directory, removing the current render if it exists:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">def render():\n\
    &apos;&apos;&apos;Render in the renderoutput directory.&apos;&apos;&apos;\n\
    output_path = os.path.join(pymxs.runtime.getDir(pymxs.runtime.Name(&quot;renderoutput&quot;)), &apos;foo.jpg&apos;)\n\
    if os.path.exists(output_path):\n\
        os.remove(output_path)\n\
    pymxs.runtime.render(outputFile=output_path)</code></pre>\n\
<h2 id=\"using-pre-and-post-render-scripts\">Using pre- and post-render scripts</h2><p>There are two mechanisms to register pre- and post-render scripts:</p>\n\
<ul>\n\
<li>set the globals <code>preRendScript</code> (string) and <code>usePreRendScript</code> (boolean), or <code>postRendScript</code> (string) and <code>usePostRendScript</code> (boolean), which correspond to the settings under <strong>Render Setup Dialog &gt; Common Parameters &gt; Scripts</strong> in the user interface.  This mechanism only works when the user presses the render button in the UI, these scripts are not called when the <code>pymxs.runtime.render()</code> function is called.</li>\n\
<li>register a #preRender or #postRender callback.  This script will be called when <code>pymxs.runtime.render()</code> is called.</li>\n\
</ul>\n\
<p>An example of the latter mechanism:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">mxs = pymxs.runtime\n\
\n\
def myPreRenderCallback():\n\
    print(&apos;PreRender Callback Called&apos;)\n\
\n\
mxs.callbacks.removeScripts(id=mxs.Name(&apos;MyCallbacks&apos;))    \n\
mxs.callbacks.addScript(mxs.name(&apos;preRender&apos;), myPreRenderCallback, id=mxs.name(&apos;MyCallbacks&apos;))\n\
# this will run the registered script first:\n\
mxs.render()</code></pre>\n\
<h2 id=\"rendering-settings-and-dialogs\">Rendering Settings and Dialogs</h2><p>You can change rendering settings using the  <code>pymxs.runtime.renderSceneDialog</code> struct.  Note that changes to render settings should be made with the dialog closed (which you can check with the <code>renderSceneDialog.isOpen()</code> function).  See the &quot;Render Scene Dialog&quot; group of topics in the MAXScript Help for information about properties and functions in this struct.</p>\n\
<p>The renderer assigned to each type of render target is held in the renderers struct.  For example, by default, renderers.production is the scanline renderer:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; pymxs.runtime.renderers.production\n\
&lt;Default_Scanline_Renderer&lt;Default_Scanline_Renderer:Default_Scanline_Renderer&gt;&gt;</code></pre>\n\
<p>The list of all available renderer classes is held in the <code>rendererClass.classes</code> list, and can be assigned to a render target:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; pymxs.runtime.renderers.current = pymxs.runtime.rendererClass.classes[5]()</code></pre>\n\
\n\
<div class=\"callout callout-note\">\n\
<div class=\"callout-title\"><i class=\"callout-icon fa fa-pencil\" aria-hidden=\"true\"></i>Note:</div>\n\
<div class=\"callout-body\">\n\
\n\
<div class=\"note note:\"><strong>Note</strong>: Because this list is accessed via Python, it is 0-based.  Also note that you create an instance of the renderer class using the constructor notation.</div>\n\
\n\
</div></div>\n\
\n\
<h2 id=\"working-with-the-viewport\">Working with the viewport</h2><p>The <code>viewport</code> struct exposes properties and methods for working with viewports.</p>\n\
<h3 id=\"iterating-through-the-list-of-viewports\">Iterating through the list of viewports</h3><p>You can use the <code>viewport.numviews</code> property to iterate through the available views.  For example, to list the viewports and their current layout types:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">mxs = pymxs.runtime\n\
for v in range(1,mxs.viewport.numviews+1):\n\
    viewtype = mxs.viewport.getType(index=v)\n\
    print(f&apos;index:{v} view type:{viewtype}&apos;)</code></pre>\n\
<div class=\"note note:\"><span class=\"notetitle\">Note:</span> We use a range of 1-numviews+1 because MAXScript is 1-based.</div>\n\
<h3 id=\"maximizing-and-zooming-a-viewport\">Maximizing and zooming a viewport</h3><p>The MAXScript command to maximize or minimize the current viewport is a &quot;Max Command&quot;, in the form of:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">max tool maximize</code></pre><p>As noted in <a href=\"./pymxs_differences/pymxs_max_commands.html\">Calling 3ds Max Commands</a>, Max commands are not directly callable in pymxs, so to invoke this command, use:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">pymxs.runtime.execute(&apos;max tool maximize&apos;)</code></pre>\n\
<p>Setting the zoom level of the current viewport is exposed through the <code>viewport.zoom()</code> function, which takes a float value setting the zoom factor, where 2.0 zooms &quot;out&quot; two times, while 0.5 zooms &quot;in&quot; two times.  If the viewport is in perspective mode, you can also use <code>viewport.zoomPerspective()</code>.</p>\n\
<p>To &quot;zoom extents&quot;, once again a Max command is used:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">max tool zoomextents\n\
max tool zoomextents all</code></pre><h3 id=\"saving-the-active-viewport-graphics-window-as-a-bitmap\">Saving the active viewport graphics window as a bitmap</h3><p>The <code>viewport.getViewportDib()</code> function returns a snapshot of a viewport&apos;s window image as bitmap.   If no additional parameters are provided, it returns a bitmap for the currently active viewport.  It takes these optional parameters:</p>\n\
<ul>\n\
<li><code>index</code> - specifies the index of the viewport to capture; the default is the currently active viewport</li>\n\
<li><code>viewPanelIndex</code> - specifies the index of the view panel to capture; the default is the currently active view panel</li>\n\
<li><code>captureAlpha</code> (boolean) - specifies whether to capture the Alpha channel as well (ARGB); the default is False</li>\n\
<li><code>gammaCorrect</code> (boolean) - specifies whether gamma correction is performed; the default is True</li>\n\
</ul>\n\
<p>For example: </p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">my_bitmap = pymxs.runtime.viewport.getViewportDib(captureAlpha=True)\n\
pymxs.runtime.display(my_bitmap)</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../using_pymxs.html\">Using pymxs</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";