var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_using_pymxs_pymxs_scenegraph_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>Working with the Scene Graph</h1></div><p>The scene graph is a tree of scene nodes.  Each node is a hub of information for objects in the scene, and is responsible for:</p>\n\
<ul>\n\
<li>applying modifiers to an object</li>\n\
<li>storing information about the geometry</li>\n\
<li>storing information about shading materials</li>\n\
<li>storing position, rotation, and scale information</li>\n\
<li>storing node tree hierarchical information: the parent and child of the node</li>\n\
</ul>\n\
<h2 id=\"traversing-the-scene-graph\">Traversing the Scene Graph</h2><p>The root node of a scene can be accessed by the MAXScript <code>rootNode</code> global.  See, for example, the <em>scene_graph.py</em> sample script:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&apos;&apos;&apos;\n\
    Creates a simple text representation of the scene graph\n\
&apos;&apos;&apos;\n\
from pymxs import runtime as rt \n\
\n\
def output_node(node, indent=&apos;&apos;):\n\
    &quot;&quot;&quot;Print the scene graph as text to stdout.&quot;&quot;&quot;\n\
    print(indent, node.Name)\n\
    for child in node.Children:\n\
        output_node(child, indent + &apos;--&apos;)\n\
\n\
output_node(rt.rootnode)</code></pre>\n\
<h2 id=\"getting-scene-objects\">Getting Scene Objects</h2><p>If you do not need the scene hierarchy information (that is, parent/child relationships), you can get a flat list of all objects in the scene with the <code>objects</code> global variable.  For example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">objs = pymxs.runtime.objects\n\
for o in objs:\n\
    print(o.name)</code></pre>\n\
<p>You can also obtain individual objects by name:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">s = pymxs.runtime.getNodeByName(&apos;Sphere002&apos;)</code></pre>\n\
<p>Finally, you can also access objects by PathName values, though this is not directly exposed to pymxs and requires a work-around.  See <a href=\"pymxs_differences/pymxs_pathname_values.html\">Accessing MAXScript PathName Values</a> for more information.</p>\n\
<h2 id=\"getting-the-selection-list\">Getting the Selection List</h2><p>There are times when it is useful to get the list of selected objects in a scene.</p>\n\
<p>There are several reserved system variables of type <code>ObjectSet</code> that contain various scene object categories.  For example, <code>objects</code> contains all the objects in the scene, <code>lights</code> all the lights, and selection contains the current <code>selection</code>.</p>\n\
<p>For example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; pymxs.runtime.selection\n\
&lt;ObjectSet&lt;$selection&gt;&gt;\n\
&gt;&gt;&gt; pymxs.runtime.selection[0]\n\
&lt;Sphere&lt;$Sphere:Sphere004 @ [90.515228,15.940669,0.000000]&gt;&gt;\n\
&gt;&gt;&gt;</code></pre>\n\
<p>Named selection sets are contained in the <code>selectionSets</code> array, and are accessed by index.</p>\n\
<p>for example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; pymxs.runtime.selectionSets[0]\n\
&lt;SelectionSet&lt;SelectionSet:my_selection&gt;&gt;</code></pre>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../using_pymxs.html\">Using pymxs</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";