var topic = "<html>\n\
<head>\n\
	<script src=\"../scripts/lib/jquery-1.11.1.min.js\" type=\"text/javascript\"/></script\n\
	<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n\
	<!--script type=\"text/javascript\" src=\"../scripts/utils/adsk.redirect.js\"/></script>	\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/lang-maxscript.js\"/></script>\n\
	<script type=\"text/javascript\" src=\"../scripts/prettify/run_prettify.js\"/></script-->\n\
    <title>3ds Max Python API Help</title>\n\
    <meta name=\"topicid\" content=\"Max_Python_API_using_pymxs_pymxs_values_html\" />\n\
    </head>\n\
<body height=\"100%\">\n\
  <a name=\"top\"></a>\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/bootstrap.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/additions.css\">\n\
  <link rel=\"stylesheet\" type=\"text/css\" href=\"/cloudhelp/2022/ENU/Max-Python-API/styles/font-awesome.min.css\">\n\
      <script src=\"/cloudhelp/2022/ENU/Max-Python-API/scripts/prettify/lang-mxs.js\"></script>\n\
\n\
  <div class=\"body_content\" id=\"body-content\">\n\
  <div class=\"head-block\"><h1>MAXScript Values in pymxs</h1></div><p>This topic covers how Python and MAXScript values and types interoperate with pymxs.  </p>\n\
<h2 id=\"maxscript-type-marshalling\">MAXScript Type Marshalling</h2><p>For simple types such as integers, floats and strings, <code>pymxs</code> has a direct reference to the data, and these values can be obtained and set directly.</p>\n\
<p>For complex types such as Array, pymxs wraps these objects with a <code>MXSWrapper</code> object, <code>pymxs.MXSWrapperBase</code>. For example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">import pymxs\n\
rt = pymxs.runtime\n\
\n\
# Create and visit a MaxScript Array\n\
gma = rt.array(*(1,2,3))\n\
print(gma)    #output: #(1,2,3)\n\
print(len(gma))    #output: 3\n\
gma[1] = 999\n\
print(gma[1])    #output: 999\n\
l = list(gma) #[1,999,3]\n\
l[2] = 888\n\
print(l)    #output: [1, 888, 3]\n\
print(gma)   # output: #(1, 999, 3)</code></pre>\n\
<p>For a demonstration of how to create, manipulate and cast complex MAXScript types in Python, see the  <em>demoPyMXSTypeInterop.py</em>  example.</p>\n\
<h2 id=\"dictionaries\">Dictionaries</h2><p>MAXScript introduced the Dictionary type in <strong>3ds Max 2017.1</strong>, and by default uses name literals for key values.  </p>\n\
<p>Consider this MAXScript dictionary:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-mxs prettyprint\">my_dict = Dictionary #(#one, 1) #(#two, 2)</code></pre>\n\
<p>In pymxs, this value is wrapped, and can be used like a MAXScript dictionary, for example getting the keys property:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">&gt;&gt;&gt; my_py_dict = pymxs.runtime.my_dict\n\
&lt;Dictionary&lt;Dictionary #name one:1 two:2 &gt;&gt;\n\
&gt;&gt;&gt; my_py_dict.keys\n\
&lt;Array&lt;#(#one, #two)&gt;&gt;</code></pre>\n\
<p>In some cases, it may be more convenient to convert the MAXScript Dictionary to a native Python Dict:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">my_py_dict = {str(key):myd[key] for key in pymxs.runtime.my_dict.keys}\n\
# yields: {&apos;two&apos;: 2, &apos;one&apos;: 1}</code></pre>\n\
<h2 id=\"maxscript-collections-in-pymxs\">MAXScript Collections in pymxs</h2><p>MAXScript arrays and dictionaries can be created and manipulated in pymxs.   In <strong>3ds Max 2021</strong> and later you can manipulate a MAXScript Array directly.  </p>\n\
<p>This example illustrates how to create and manipulate MAXScript arrays and dictionaries in pymxs:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\"># MAXScript Collections in pymxs\n\
\n\
from pymxs import runtime as rt\n\
import pymxs\n\
\n\
py_array = rt.Array(1,2,3)\n\
# 3ds Max 2020 and earlier requires rt.append\n\
rt.append(py_array, 4)\n\
\n\
try:\n\
    # In 3ds Max 2021, we can do this directly:\n\
    # adding an arbitrary element will created &quot;undefined&quot; intermediary elements:\n\
    py_array[5] = 5\n\
except:\n\
    print(&apos;Direct access is available in 3ds Max 2021+&apos;)\n\
\n\
py_dict = rt.Dictionary((&quot;one&quot;,1), (&quot;two&quot;,2))\n\
py_dict[&quot;three&quot;] = 3\n\
# you could also use rt.SetDictValue(py_dict, &apos;three&apos;, 3)\n\
\n\
print(&quot;py_array: &quot;, py_array)\n\
print(&quot;py_dict: &quot;, py_dict)</code></pre>\n\
<p>Results:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"prettyprint\">py_array:  #(1, 2, 3, 4, undefined, 5)\n\
py_dict:  Dictionary #string (DataPair &quot;three&quot; 3) (DataPair &quot;one&quot; 1) (DataPair &quot;two&quot; 2) </code></pre><h2 id=\"using-pymxs-objects-as-python-dictionary-keys\">Using pymxs objects as Python Dictionary keys</h2><p>As of 3ds Max 2019 Update 1, you can use some pymxs objects (such as scene nodes) as key values in native Python dictionaries (though not in MAXScript Dictionaries).  For example:</p>\n\
<pre class=\"pre codeblock prettyprint\"><code class=\"language-py prettyprint\">from pymxs import runtime as rt\n\
\n\
t1 = rt.teapot()\n\
t2 = rt.teapot(pos=rt.point3(20,20,0))\n\
t2.parent = t1\n\
\n\
node_dict = {t1:&quot;Teapot 1&quot;, t2:&quot;Teapot2&quot;}\n\
\n\
print(node_dict.keys())\n\
print(node_dict.values())</code></pre>\n\
<p>To determine whether a MAXScript object can be used as a Python dictionary key, you can check whether it implements the python <code>__hash__()</code> function.</p>\n\
\n\
  </div>\n\
\n\
    <div class=\"related-links\">\n\
  \n\
    <div class=\"familylinks\">\n\
      <div class=\"parentLink\">\n\
        <strong>Parent page:</strong> <a class=\"link\" href=\"../using_pymxs.html\">Using pymxs</a>\n\
      </div>\n\
  \n\
    </div>\n\
  \n\
    </div>\n\
\n\
</body>\n\
</html>\n\
";